//
//  CSCommentItem.m
//  C64SceneBrowser
//
//  Created by Andreas Varga on 20.03.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "CSCommentItem.h"
#import "CSUserItem.h"
#import "CSScenerItem.h"
#import "NSString+HTML.h"
#import "NSAttributedString+Attributes.h"


@implementation CSCommentItem

@synthesize date = _date;
@synthesize text = _text;
@synthesize user = _user;
@synthesize imageOffsets = _imageOffsets;
@synthesize imageUrls = _imageUrls;


// ----------------------------------------------------------------------------
- (id)init
// ----------------------------------------------------------------------------
{
    self = [super init];
    if (self)
    {
        _date = nil;
        _text = nil;
        _user = nil;
    }
    return self;
}


// ----------------------------------------------------------------------------
- (id) initWithXMLElement:(TBXMLElement*)commentElement
// ----------------------------------------------------------------------------
{
    self = [self init];
    if (self)
    {
        [self setFromXMLElement:commentElement];
    }
    return self;
}


// ----------------------------------------------------------------------------
- (void) setFromXMLElement:(TBXMLElement*)commentElement
// ----------------------------------------------------------------------------
{
    _date = [TBXML textForElement:[TBXML childElementNamed:@"Date" parentElement:commentElement] withStringEncoding:NSUTF8StringEncoding];
    
    TBXMLElement* userElement = [TBXML childElementNamed:@"CSDbUser" parentElement:commentElement];
    NSAssert(userElement != NULL, @"Comment must have user!");
    
    _user = [[CSUserItem alloc] initWithXMLElement:userElement];
    
    TBXMLElement* entryElement = [TBXML childElementNamed:@"CSDbEntry" parentElement:userElement];
    
    if (entryElement != NULL)
    {
        TBXMLElement* handleElement = [TBXML childElementNamed:@"Handle" parentElement:entryElement];
        if (handleElement != NULL)
            _user.scener = [[CSScenerItem alloc] initWithXMLElement:handleElement];
    }
    
    _imageOffsets = [NSMutableArray array];
    _imageUrls = [NSMutableArray array];
    
    NSString* text = [TBXML textForElement:[TBXML childElementNamed:@"Text" parentElement:commentElement] withStringEncoding:NSWindowsCP1252StringEncoding];
    
    text = [[text stringByReplacingOccurrencesOfString:@"\r\n" withString:@"\n"] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
    text = [text stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
    text = [text stringByDecodingHTMLEntities];
    text = [self replaceShortLinksWithUrlSchemeInString:text forType:@"release"];
    text = [self replaceShortLinksWithUrlSchemeInString:text forType:@"scener"];
    text = [self replaceShortLinksWithUrlSchemeInString:text forType:@"event"];
    text = [self replaceShortLinksWithUrlSchemeInString:text forType:@"group"];
    text = [self replaceShortLinksWithUrlSchemeInString:text forType:@"sid"];
    
    text = [text stringByReplacingOccurrencesOfString:@"http://noname.c64.org/csdb/release/?id=" withString:@"csdb://release/" options:NSCaseInsensitiveSearch range:NSMakeRange(0, text.length)];
    text = [text stringByReplacingOccurrencesOfString:@"http://noname.c64.org/csdb/scener/?id=" withString:@"csdb://scener/" options:NSCaseInsensitiveSearch range:NSMakeRange(0, text.length)];
    text = [text stringByReplacingOccurrencesOfString:@"http://noname.c64.org/csdb/event/?id=" withString:@"http://csdb.dk/event/?id=" options:NSCaseInsensitiveSearch range:NSMakeRange(0, text.length)];
    text = [text stringByReplacingOccurrencesOfString:@"http://noname.c64.org/csdb/group/?id=" withString:@"http://csdb.dk/group/?id=" options:NSCaseInsensitiveSearch range:NSMakeRange(0, text.length)];
    text = [text stringByReplacingOccurrencesOfString:@"http://noname.c64.org/csdb/sid/?id=" withString:@"http://csdb.dk/sid/?id=" options:NSCaseInsensitiveSearch range:NSMakeRange(0, text.length)];
    
    
//    NSRange linkRange = NSMakeRange(released_text.length - group.name.length, group.name.length);
//    [linkRanges addObject:[NSValue valueWithRange:linkRange]];
//    [linkURLs addObject:[NSURL URLWithString:group.website]];

    
    //NSMutableArray* urlRanges = [NSMutableArray array];

    NSRange url_start_range = [text rangeOfString:@"[url" options:NSCaseInsensitiveSearch];
    while (url_start_range.location != NSNotFound)
    {
        NSString* url = nil;
        NSUInteger pos_after_range = url_start_range.location + url_start_range.length;
        NSRange remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
        
        // Find closing bracket of url start tag
        NSRange url_close_range = [text rangeOfString:@"]" options:NSLiteralSearch range:remaining_range];
        if (url_close_range.location != NSNotFound)
        {
            // Check if the url is not inside the tag, and extract it
            NSRange url_address_range = NSMakeRange(url_start_range.location + 4, url_close_range.location - (url_start_range.location + 4));
            if (url_address_range.length > 1)
            {
                // Remove =
                url_address_range.location += 1;
                url_address_range.length -= 1;
                url = [text substringWithRange:url_address_range];
            }
            
            pos_after_range = url_close_range.location + url_close_range.length;
            remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
            
            NSRange url_end_range = [text rangeOfString:@"[/url]" options:NSCaseInsensitiveSearch range:remaining_range];
            if (url_end_range.location == NSNotFound)
                url_end_range = NSMakeRange(text.length-1, 0);
                
            // Extract url text
            if (url == nil)
            {
                NSUInteger url_text_pos = url_close_range.location + 1;
                NSRange url_text_range = NSMakeRange(url_text_pos, url_end_range.location - url_text_pos);
                url = [text substringWithRange:url_text_range];
            }
            
            NSRange full_url_range = NSMakeRange(url_start_range.location, (url_end_range.location + url_end_range.length) - url_start_range.location);
            text = [text stringByReplacingCharactersInRange:full_url_range withString:url];
            
            NSRange replaced_range = NSMakeRange(url_start_range.location, [url length]);

            //NSValue* rangeValue = [NSValue valueWithRange:replaced_range];
            //[urlRanges addObject:rangeValue];
            
            pos_after_range = replaced_range.location + replaced_range.length;
            remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
            url_start_range = [text rangeOfString:@"[url" options:NSCaseInsensitiveSearch range:remaining_range];
        }
        else
            break;
    }

    NSRange img_start_range = [text rangeOfString:@"[img]" options:NSCaseInsensitiveSearch];
    while (img_start_range.location != NSNotFound)
    {
        NSUInteger pos_after_range = img_start_range.location + img_start_range.length;
        NSRange remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
        
        NSRange img_end_range = [text rangeOfString:@"[/img]" options:NSCaseInsensitiveSearch range:remaining_range];
        if (img_end_range.location != NSNotFound)
        {
            // Extract url text
            NSUInteger img_text_pos = pos_after_range;
            NSRange img_text_range = NSMakeRange(img_text_pos, img_end_range.location - img_text_pos);
            NSString* url = [text substringWithRange:img_text_range];
            [_imageUrls addObject:url];
            
            NSRange full_img_range = NSMakeRange(img_start_range.location, (img_end_range.location + img_end_range.length) - img_start_range.location);
            
            NSString* placeholder = @"\n\n\n\n\n\n";
            NSUInteger image_offset = full_img_range.location;
            if (full_img_range.location > 0)
            {
                placeholder = @"\n\n\n\n\n\n\n";
                image_offset += 1;  // +1 to include first \n that was replaced
            }

            text = [text stringByReplacingCharactersInRange:full_img_range withString:placeholder];
            [_imageOffsets addObject:[NSNumber numberWithUnsignedInteger:image_offset]];
            
            NSRange replaced_range = NSMakeRange(img_start_range.location, [placeholder length]);
            pos_after_range = replaced_range.location + replaced_range.length;
            remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
            img_start_range = [text rangeOfString:@"[img]" options:NSCaseInsensitiveSearch range:remaining_range];
        }
        else
            break;
    }
    
    
    NSMutableArray* codeRanges = [NSMutableArray array];
    NSRange code_start_range = [text rangeOfString:@"[code]" options:NSCaseInsensitiveSearch];
    while (code_start_range.location != NSNotFound)
    {
        NSUInteger pos_after_range = code_start_range.location + code_start_range.length;
        NSRange remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
        
        NSRange code_end_range = [text rangeOfString:@"[/code]" options:NSCaseInsensitiveSearch range:remaining_range];
        if (code_end_range.location != NSNotFound)
        {
            // Extract code text
            NSUInteger code_text_pos = pos_after_range;
            NSRange code_text_range = NSMakeRange(code_text_pos, code_end_range.location - code_text_pos);
            NSString* code_text = [text substringWithRange:code_text_range];
            
            NSRange full_code_range = NSMakeRange(code_start_range.location, (code_end_range.location + code_end_range.length) - code_start_range.location);
            text = [text stringByReplacingCharactersInRange:full_code_range withString:code_text];

            NSRange replaced_range = NSMakeRange(code_start_range.location, [code_text length]);
            NSValue* rangeValue = [NSValue valueWithRange:replaced_range];
            [codeRanges addObject:rangeValue];
            
            pos_after_range = replaced_range.location + replaced_range.length;
            remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
            code_start_range = [text rangeOfString:@"[code]" options:NSCaseInsensitiveSearch range:remaining_range];
        }
        else
            break;
    }
    
    NSMutableArray* quoteRanges = [NSMutableArray array];
    
    NSRange quote_start_range = [text rangeOfString:@"[quote" options:NSCaseInsensitiveSearch];
    while (quote_start_range.location != NSNotFound)
    {
        NSString* quote_name = nil;
        NSUInteger pos_after_range = quote_start_range.location + quote_start_range.length;
        NSRange remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
        
        // Find closing bracket of quote start tag
        NSRange quote_close_range = [text rangeOfString:@"]" options:NSLiteralSearch range:remaining_range];
        if (quote_close_range.location != NSNotFound)
        {
            // Check if the quote has a name, and extract it
            NSRange name_range = NSMakeRange(quote_start_range.location + 6, quote_close_range.location - (quote_start_range.location + 6));
            if (name_range.length > 1)
            {
                // Remove =
                name_range.location += 1;
                name_range.length -= 1;
                quote_name = [text substringWithRange:name_range];
            }
            
            pos_after_range = quote_close_range.location + quote_close_range.length;
            remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
            
            NSRange quote_end_range = [text rangeOfString:@"[/quote]" options:NSCaseInsensitiveSearch range:remaining_range];
            if (quote_end_range.location != NSNotFound)
            {
                
                // Extract quote text
                NSUInteger quote_text_pos = quote_close_range.location + 1;
                NSRange quote_text_range = NSMakeRange(quote_text_pos, quote_end_range.location - quote_text_pos);
                NSString* quote_text = [text substringWithRange:quote_text_range];
                
                // Build quote text
                NSString* new_quote = nil;
                if (quote_name != nil)
                    new_quote = [NSString stringWithFormat:@"Quoting %@:\n\t%@\n\n", quote_name, quote_text];
                else
                    new_quote = [NSString stringWithFormat:@"Quote:\n\t%@\n\n", quote_text];
                
                NSRange full_quote_range = NSMakeRange(quote_start_range.location, (quote_end_range.location + quote_end_range.length) - quote_start_range.location);
                text = [text stringByReplacingCharactersInRange:full_quote_range withString:new_quote];
                
                NSRange replaced_range = NSMakeRange(quote_start_range.location, [new_quote length]);
                NSValue* rangeValue = [NSValue valueWithRange:replaced_range];
                [quoteRanges addObject:rangeValue];
                
                pos_after_range = replaced_range.location + replaced_range.length;
                remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
                quote_start_range = [text rangeOfString:@"[quote" options:NSCaseInsensitiveSearch range:remaining_range];
            }
            else
                break;
        }
        else
            break;
    }
    
    
    NSMutableArray* boldRanges = [NSMutableArray array];
    NSRange bold_start_range = [text rangeOfString:@"[b]" options:NSCaseInsensitiveSearch];
    while (bold_start_range.location != NSNotFound)
    {
        NSUInteger pos_after_range = bold_start_range.location + bold_start_range.length;
        NSRange remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
        
        NSRange bold_end_range = [text rangeOfString:@"[/b]" options:NSCaseInsensitiveSearch range:remaining_range];
        if (bold_end_range.location != NSNotFound)
        {
            NSUInteger bold_text_pos = pos_after_range;
            NSRange bold_text_range = NSMakeRange(bold_text_pos, bold_end_range.location - bold_text_pos);
            NSString* bold_text = [text substringWithRange:bold_text_range];
            
            NSRange full_bold_range = NSMakeRange(bold_start_range.location, (bold_end_range.location + bold_end_range.length) - bold_start_range.location);
            text = [text stringByReplacingCharactersInRange:full_bold_range withString:bold_text];

            NSRange replaced_range = NSMakeRange(bold_start_range.location, [bold_text length]);
            NSValue* rangeValue = [NSValue valueWithRange:replaced_range];
            [boldRanges addObject:rangeValue];
            
            pos_after_range = replaced_range.location + replaced_range.length;
            remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
            bold_start_range = [text rangeOfString:@"[b]" options:NSCaseInsensitiveSearch range:remaining_range];
        }
        else
            break;
    }
    
    NSMutableArray* italicRanges = [NSMutableArray array];
    NSRange italic_start_range = [text rangeOfString:@"[i]" options:NSCaseInsensitiveSearch];
    while (italic_start_range.location != NSNotFound)
    {
        NSUInteger pos_after_range = italic_start_range.location + italic_start_range.length;
        NSRange remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
        
        NSRange italic_end_range = [text rangeOfString:@"[/i]" options:NSCaseInsensitiveSearch range:remaining_range];
        if (italic_end_range.location != NSNotFound)
        {
            NSUInteger italic_text_pos = pos_after_range;
            NSRange italic_text_range = NSMakeRange(italic_text_pos, italic_end_range.location - italic_text_pos);
            NSString* italic_text = [text substringWithRange:italic_text_range];
            
            NSRange full_italic_range = NSMakeRange(italic_start_range.location, (italic_end_range.location + italic_end_range.length) - italic_start_range.location);
            text = [text stringByReplacingCharactersInRange:full_italic_range withString:italic_text];
            
            NSRange replaced_range = NSMakeRange(italic_start_range.location, [italic_text length]);
            NSValue* rangeValue = [NSValue valueWithRange:replaced_range];
            [italicRanges addObject:rangeValue];
            
            pos_after_range = replaced_range.location + replaced_range.length;
            remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
            italic_start_range = [text rangeOfString:@"[i]" options:NSCaseInsensitiveSearch range:remaining_range];
        }
        else
            break;
    }

    NSMutableArray* strikeRanges = [NSMutableArray array];
    NSRange strike_start_range = [text rangeOfString:@"[s]" options:NSCaseInsensitiveSearch];
    while (strike_start_range.location != NSNotFound)
    {
        NSUInteger pos_after_range = strike_start_range.location + strike_start_range.length;
        NSRange remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
        
        NSRange strike_end_range = [text rangeOfString:@"[/s]" options:NSCaseInsensitiveSearch range:remaining_range];
        if (strike_end_range.location != NSNotFound)
        {
            NSUInteger strike_text_pos = pos_after_range;
            NSRange strike_text_range = NSMakeRange(strike_text_pos, strike_end_range.location - strike_text_pos);
            NSString* strike_text = [text substringWithRange:strike_text_range];
            
            NSRange full_strike_range = NSMakeRange(strike_start_range.location, (strike_end_range.location + strike_end_range.length) - strike_start_range.location);
            text = [text stringByReplacingCharactersInRange:full_strike_range withString:strike_text];
            
            NSRange replaced_range = NSMakeRange(strike_start_range.location, [strike_text length]);
            NSValue* rangeValue = [NSValue valueWithRange:replaced_range];
            [strikeRanges addObject:rangeValue];
            
            pos_after_range = replaced_range.location + replaced_range.length;
            remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
            strike_start_range = [text rangeOfString:@"[s]" options:NSCaseInsensitiveSearch range:remaining_range];
        }
        else
            break;
    }

    NSMutableAttributedString* attributed_text = [NSMutableAttributedString attributedStringWithString:text];
    [attributed_text setTextColor:[UIColor whiteColor]];
    [attributed_text setFont:[UIFont systemFontOfSize:13.0f]];
    
//    for (NSValue* value in urlRanges)
//    {
//        NSRange range = [value rangeValue];
//        [attributed_text addAttribute:NSLinkAttributeName value:[NSURL URLWithString:[text substringWithRange:range]] range:range];
//    }
    
    for (NSValue* value in quoteRanges)
    {
        NSRange range = [value rangeValue];
        [attributed_text setTextColor:[UIColor colorWithWhite:0.5f alpha:1.0f] range:range];
    }

    for (NSValue* value in codeRanges)
    {
        NSRange range = [value rangeValue];
        [attributed_text setFont:[UIFont fontWithName:@"Courier" size:13.0f] range:range];
    }
    
    for (NSValue* value in boldRanges)
    {
        NSRange range = [value rangeValue];
        [attributed_text setFont:[UIFont boldSystemFontOfSize:13.0f] range:range];
    }
    
    for (NSValue* value in italicRanges)
    {
        NSRange range = [value rangeValue];
        [attributed_text setFont:[UIFont italicSystemFontOfSize:13.0f] range:range];
    }

    for (NSValue* value in strikeRanges)
    {
        NSRange range = [value rangeValue];
        [attributed_text removeAttribute:NSStrikethroughStyleAttributeName range:range];
        [attributed_text addAttribute:NSStrikethroughStyleAttributeName value:[NSNumber numberWithInteger:1] range:range];
    }

    _text = attributed_text;
}


- (NSString*) replaceShortLinksWithUrlSchemeInString:(NSString*)text forType:(NSString*)type
{
    NSString* open_tag = [NSString stringWithFormat:@"<%@ id=", type];
    NSRange tag_open_range = [text rangeOfString:open_tag options:NSCaseInsensitiveSearch];
    while (tag_open_range.location != NSNotFound)
    {
        NSString* item_id = nil;
        NSUInteger pos_after_range = tag_open_range.location + tag_open_range.length;
        NSRange remaining_range = NSMakeRange(pos_after_range, [text length] - pos_after_range);
        
        // Find closing bracket of short link tag
        NSRange tag_close_range = [text rangeOfString:@">" options:NSLiteralSearch range:remaining_range];
        if (tag_close_range.location != NSNotFound)
        {
            // Check if the quote has a name, and extract it
            NSRange id_range = NSMakeRange(tag_open_range.location + [open_tag length], tag_close_range.location - (tag_open_range.location + [open_tag length]));
            if (id_range.length > 1)
            {
                item_id = [text substringWithRange:id_range];
                
                // Strip quotes
                item_id = [item_id stringByReplacingOccurrencesOfString:@"\"" withString:@"" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [item_id length])];

                // Replace full tag range with URL
                NSString* url = [NSString stringWithFormat:@"csdb://%@/%@", type, item_id];
                
                NSUInteger pos_after_close_range = tag_close_range.location + tag_close_range.length;
                NSRange full_tag_range = NSMakeRange(tag_open_range.location, pos_after_close_range - tag_open_range.location);
                
                text = [text stringByReplacingCharactersInRange:full_tag_range withString:url];
            }
            else
                break;
        }
        else
            break;
        
        tag_open_range = [text rangeOfString:open_tag options:NSCaseInsensitiveSearch];
    }
    
    return text;
}


@end
