//
//  CSDatabaseItem.m
//  C64SceneBrowser
//
//  Created by Andreas Varga on 15.05.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "CSDatabaseItem.h"
#import "CSDatabaseCache.h"
#import "CSReleaseItem.h"
#import "CSScenerItem.h"
#import "CSGroupItem.h"
#import "CSEventItem.h"
#import "TBXML.h"


@implementation CSDatabaseItem


@synthesize identifier = _identifier;
@synthesize cached_depth = _cached_depth;
@synthesize requested_depth = _requested_depth;


// ----------------------------------------------------------------------------
- (id) init
// ----------------------------------------------------------------------------
{
    self = [super init];
    if (self)
    {
        _identifier = nil;
        _cached_depth = 0;
        _owner = nil;
        _requested_depth = 0;
        
        _data = nil;
        _urlConnection = nil;
    }
    return self;
    
}


// ----------------------------------------------------------------------------
- (id) initWithIdentifier:(NSString*)identifier
// ----------------------------------------------------------------------------
{
    self = [super init];
    if (self)
    {
        _identifier = identifier;
        _cached_depth = 0;
        _owner = nil;
        _requested_depth = 0;

        _data = nil;
        _urlConnection = nil;
    }
    return self;
    
}


// ----------------------------------------------------------------------------
- (CSDatabaseItemType) type
// ----------------------------------------------------------------------------
{
    return CSDB_BASE;
}


// ----------------------------------------------------------------------------
- (void) cacheAndNotify:(id)owner withDepth:(NSUInteger)depth
// ----------------------------------------------------------------------------
{
    _owner = owner;
    _requested_depth = depth;
    _cached_depth = 0;
    
    NSString* url_string = nil;
    
    switch ([self type])
    {
        case CSDB_RELEASE:
            url_string = [NSString stringWithFormat:@"http://csdb.dk/webservice/?type=release&id=%@&depth=%d", _identifier, (unsigned int)depth];
            break;
            
        case CSDB_SCENER:
            url_string = [NSString stringWithFormat:@"http://csdb.dk/webservice/?type=scener&id=%@&depth=%d", _identifier, (unsigned int)depth];
            break;
            
        case CSDB_GROUP:
            url_string = [NSString stringWithFormat:@"http://csdb.dk/webservice/?type=group&id=%@&depth=%d", _identifier, (unsigned int)depth];
            break;
            
        case CSDB_EVENT:
            url_string = [NSString stringWithFormat:@"http://csdb.dk/webservice/?type=event&id=%@&depth=%d", _identifier, (unsigned int)depth];
            break;

        default:
            NSAssert(false, @"cacheAndNotify with invalid type");
    }

    NSURL* url = [NSURL URLWithString:url_string];
    NSMutableURLRequest* request = [NSMutableURLRequest requestWithURL:url cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:60.0];
    _data = [NSMutableData data];
    _urlConnection = [[NSURLConnection alloc] initWithRequest:request delegate:self];
}


// ----------------------------------------------------------------------------
- (void) setFromXMLData:(NSData*)xmlData
// ----------------------------------------------------------------------------
{
    _cached_depth = _requested_depth;
}


// ----------------------------------------------------------------------------
- (void) connection:(NSURLConnection*)connection didReceiveResponse:(NSURLResponse*)response
// ----------------------------------------------------------------------------
{
    if (connection == _urlConnection)
    {
        [_data setLength:0];
        return;
    }
}


// ----------------------------------------------------------------------------
- (void) connection:(NSURLConnection*)connection didReceiveData:(NSData*)data
// ----------------------------------------------------------------------------
{
    if (connection == _urlConnection)
    {
        [_data appendData:data];
        return;
    }
    
}


// ----------------------------------------------------------------------------
- (void) connection:(NSURLConnection*)connection didFailWithError:(NSError*)error
// ----------------------------------------------------------------------------
{
    //NSLog(@"connection failed\n");
    
    // Check if we have a cached result for this item
    CSDatabaseItem* item = [[CSDatabaseCache sharedInstance] itemForType:[self type] withIdentifier:_identifier];
    BOOL is_in_cache = [[CSDatabaseCache sharedInstance] readItem:item];
    
    if (is_in_cache)
    {
        if (_owner != nil)
            [_owner databaseItemIsCached:self];
    }
    else
    {
        if (_owner != nil)
            [_owner cachingFailed];
    }
}


// ----------------------------------------------------------------------------
- (void) connectionDidFinishLoading:(NSURLConnection*)connection
// ----------------------------------------------------------------------------
{
    if (connection == _urlConnection && [_data length] > 0)
    {
        //NSLog(@"Request for release %@ finished\n", _identifier);
        
        //NSLog(@"Caching of item %@ %@ finished\n", self, _identifier);

        [self setFromXMLData:_data];
        
        // All image downloads submitted
        
        if (_cached_depth == 2)
            [[CSDatabaseCache sharedInstance] writeItem:self withData:_data];
        
        //NSLog(@"Parsing of item %@ %@ finished\n", self, _identifier);
        
        if (_owner != nil)
            [_owner databaseItemIsCached:self];
    }
}



@end
