//
//  CSEventItem.m
//  C64SceneBrowser
//
//  Created by Andreas Varga on 22.04.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "CSEventItem.h"
#import "NSString+HTML.h"

@implementation CSEventItem


@synthesize name = _name;
@synthesize alternativeName = _alternativeName;
@synthesize tagline = _tagline;
@synthesize startDate = _startDate;
@synthesize endDate = _endDate;


// ----------------------------------------------------------------------------
- (id)init
// ----------------------------------------------------------------------------
{
    self = [super init];
    if (self)
    {
        _name = nil;
        _alternativeName = nil;
        _tagline = nil;
        _startDate = [[NSDateComponents alloc] init];
        _endDate = [[NSDateComponents alloc] init];
    }
    return self;
}


// ----------------------------------------------------------------------------
- (id) initWithEventId:(NSString*)eventId
// ----------------------------------------------------------------------------
{
    self = [super initWithIdentifier:eventId];
    if (self)
    {
        _name = nil;
        _alternativeName = nil;
        _tagline = nil;
        _startDate = [[NSDateComponents alloc] init];
        _endDate = [[NSDateComponents alloc] init];
    }
    return self;
}


// ----------------------------------------------------------------------------
- (CSDatabaseItemType) type
// ----------------------------------------------------------------------------
{
    return CSDB_EVENT;
}


// ----------------------------------------------------------------------------
- (id) initWithXMLElement:(TBXMLElement*)eventElement
// ----------------------------------------------------------------------------
{
    self = [self init];
    if (self)
    {
        [self setFromXMLElement:eventElement];
    }
    return self;
}


// ----------------------------------------------------------------------------
- (void) setFromXMLElement:(TBXMLElement*)eventElement
// ----------------------------------------------------------------------------
{
    CSEventItem* event = self;
    
    event.identifier = [TBXML textForElement:[TBXML childElementNamed:@"ID" parentElement:eventElement]];
    TBXMLElement* nameElement = [TBXML childElementNamed:@"Name" parentElement:eventElement];
    if (nameElement != NULL)
    {
        event.name = [TBXML textForElement:nameElement withStringEncoding:NSUTF8StringEncoding];
        event.name = [event.name stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
        event.name = [event.name stringByDecodingHTMLEntities];
    }
    
    TBXMLElement* akaElement = [TBXML childElementNamed:@"AKA" parentElement:eventElement];
    if (akaElement != NULL)
    {
        event.alternativeName = [TBXML textForElement:akaElement withStringEncoding:NSUTF8StringEncoding];
        event.alternativeName = [event.alternativeName stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
        event.alternativeName = [event.alternativeName stringByDecodingHTMLEntities];
    }

    TBXMLElement* taglineElement = [TBXML childElementNamed:@"Tagline" parentElement:eventElement];
    if (taglineElement != NULL)
    {
        event.tagline = [TBXML textForElement:taglineElement withStringEncoding:NSUTF8StringEncoding];
        event.tagline = [event.tagline stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
        event.tagline = [event.tagline stringByDecodingHTMLEntities];
    }
    
    NSString* startDay = [TBXML textForElement:[TBXML childElementNamed:@"StartDay" parentElement:eventElement] withStringEncoding:NSUTF8StringEncoding];
    NSString* startMonth = [TBXML textForElement:[TBXML childElementNamed:@"StartMonth" parentElement:eventElement] withStringEncoding:NSUTF8StringEncoding];
    NSString* startYear = [TBXML textForElement:[TBXML childElementNamed:@"StartYear" parentElement:eventElement] withStringEncoding:NSUTF8StringEncoding];
    
    if (startDay != nil && startDay.length > 0)
        [_startDate setDay:[startDay integerValue]];
    
    if (startMonth != nil && startMonth.length > 0)
        [_startDate setMonth:[startMonth integerValue]];
    
    if (startYear != nil && startYear.length > 0)
        [_startDate setYear:[startYear integerValue]];

    NSString* endDay = [TBXML textForElement:[TBXML childElementNamed:@"EndDay" parentElement:eventElement] withStringEncoding:NSUTF8StringEncoding];
    NSString* endMonth = [TBXML textForElement:[TBXML childElementNamed:@"EndMonth" parentElement:eventElement] withStringEncoding:NSUTF8StringEncoding];
    NSString* endYear = [TBXML textForElement:[TBXML childElementNamed:@"EndYear" parentElement:eventElement] withStringEncoding:NSUTF8StringEncoding];
    
    if (endDay != nil && endDay.length > 0)
        [_endDate setDay:[endDay integerValue]];
    
    if (endMonth != nil && endMonth.length > 0)
        [_endDate setMonth:[endMonth integerValue]];
    
    if (endYear != nil && endYear.length > 0)
        [_endDate setYear:[endYear integerValue]];
}



// ----------------------------------------------------------------------------
- (NSComparisonResult) compareStartDates:(CSEventItem*)otherItem
// ----------------------------------------------------------------------------
{
    if (_startDate == nil || otherItem.startDate == nil)
        return NSOrderedSame;
    
    NSCalendar* calendar = [[NSCalendar alloc] initWithCalendarIdentifier:NSCalendarIdentifierGregorian];
    NSDate* date1 = [calendar dateFromComponents:_startDate];
    NSDate* date2 = [calendar dateFromComponents:otherItem.startDate];
    
    return [date2 compare:date1];
}


@end

