//
//  CSReleaseItem.m
//  C64SceneBrowser
//
//  Created by Andreas Varga on 17.03.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "CSReleaseItem.h"
#import "CSCommentItem.h"
#import "CSUserItem.h"
#import "CSScenerItem.h"
#import "CSCreditItem.h"
#import "CSGroupItem.h"
#import "CSEventItem.h"
#import "CSDatabaseCache.h"
#import "TBXML.h"
#import "NSString+HTML.h"


@implementation CSReleaseItem

@synthesize is_loadmore_item = _is_loadmore_item;
@synthesize name = _name;
@synthesize releaseType = _releaseType;
@synthesize releaseDate = _releaseDate;
@synthesize screenshot = _screenshot;
@synthesize rating = _rating;
@synthesize releaseGroups = _releaseGroups;
@synthesize releaseSceners = _releaseSceners;
@synthesize releasedAtEvent = _releasedAtEvent;
@synthesize achievement = _achievement;

@synthesize creditsDictionary = _creditsDictionary;
@synthesize creditTypes = _creditTypes;
@synthesize comments = _comments;
@synthesize summaries = _summaries;
@synthesize productionNotes = _productionNotes;
@synthesize goofs = _goofs;
@synthesize hiddenParts = _hiddenParts;
@synthesize trivias = _trivias;
@synthesize downloadURLs = _downloadURLs;


// ----------------------------------------------------------------------------
- (id) init
// ----------------------------------------------------------------------------
{
    self = [super init];
    if (self)
    {
        _is_loadmore_item = NO;
        _name = nil;
        _releaseType = nil;
        _releaseDate = [[NSDateComponents alloc] init];
        _screenshot = nil;
        _rating = nil;
        _releasedAtEvent = nil;
        _achievement = nil;
        
        _releaseGroups = [NSMutableArray array];
        _releaseSceners = [NSMutableArray array]; 
        _comments = [NSMutableArray array];
        _summaries = [NSMutableArray array];
        _productionNotes = [NSMutableArray array];
        _goofs = [NSMutableArray array];
        _hiddenParts = [NSMutableArray array];
        _trivias = [NSMutableArray array];
        _downloadURLs = [NSMutableArray array];
        
        _creditsDictionary = [NSMutableDictionary dictionary];
        _creditTypes = [NSMutableArray array];
    }
    return self;
}


// ----------------------------------------------------------------------------
- (id) initWithReleaseId:(NSString*)releaseId
// ----------------------------------------------------------------------------
{
    self = [super initWithIdentifier:releaseId];
    if (self)
    {
        _is_loadmore_item = NO;
        _name = nil;
        _releaseType = nil;
        _releaseDate = [[NSDateComponents alloc] init];
        _screenshot = nil;
        _rating = nil;
        _releasedAtEvent = nil;
        _achievement = nil;
        
        _releaseGroups = [NSMutableArray array];
        _releaseSceners = [NSMutableArray array]; 
        _comments = [NSMutableArray array];
        _summaries = [NSMutableArray array];
        _productionNotes = [NSMutableArray array];
        _goofs = [NSMutableArray array];
        _hiddenParts = [NSMutableArray array];
        _trivias = [NSMutableArray array];
        _downloadURLs = [NSMutableArray array];
        
        _creditsDictionary = [NSMutableDictionary dictionary];
        _creditTypes = [NSMutableArray array];
    }
    return self;
}


// ----------------------------------------------------------------------------
- (CSDatabaseItemType) type
// ----------------------------------------------------------------------------
{
    return CSDB_RELEASE;
}


// ----------------------------------------------------------------------------
- (id) initWithXMLElement:(TBXMLElement*)releaseElement
// ----------------------------------------------------------------------------
{
    self = [self init];
    if (self)
    {
        self = [self setFromXMLElement:releaseElement];
    }
    return self;
}


// ----------------------------------------------------------------------------
- (void) cacheAndNotify:(id)owner withDepth:(NSUInteger)depth
// ----------------------------------------------------------------------------
{
    //NSLog(@"Caching release %@, depth %d\n", self.identifier, depth);
    
    [self clear];
    [super cacheAndNotify:owner withDepth:depth];
}


// ----------------------------------------------------------------------------
- (void) clear
// ----------------------------------------------------------------------------
{
    [_releaseGroups removeAllObjects];
    [_releaseSceners removeAllObjects];
    [_comments removeAllObjects];
    [_summaries removeAllObjects];
    [_productionNotes removeAllObjects];
    [_goofs removeAllObjects];
    [_hiddenParts removeAllObjects];
    [_trivias removeAllObjects];
    
    [_creditsDictionary removeAllObjects];
    [_creditTypes removeAllObjects];
}


// ----------------------------------------------------------------------------
- (void) setFromXMLData:(NSData*)xmlData
// ----------------------------------------------------------------------------
{
    [self clear];
    
    NSError* error;
    TBXML* tbxml = [TBXML tbxmlWithXMLData:xmlData error:&error];
    if (tbxml == nil || tbxml.rootXMLElement == NULL)
        return;
    
    TBXMLElement* releaseElement = [TBXML childElementNamed:@"Release" parentElement:tbxml.rootXMLElement];
    if (releaseElement == NULL)
        return;
    [self setFromXMLElement:releaseElement];
    
    [super setFromXMLData:xmlData];
}


/*
void LogIt (NSString *format, ...)
{
    va_list args;
    va_start (args, format);
    NSString *string;
    string = [[NSString alloc] initWithFormat: format  arguments: args];
    va_end (args);
    printf ("%s", [string cStringUsingEncoding:NSUTF8StringEncoding]);
}


- (void) dumpTree:(TBXMLElement*)element
{
    NSString* text = [[TBXML textForElement:element] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
    if ([text length] > 0)
        LogIt(@"<%@>%@", [TBXML elementName:element], text);
    else
        LogIt(@"<%@>\n", [TBXML elementName:element]);
    
    TBXMLElement* child = element->firstChild;
    while (child != NULL)
    {
        [self dumpTree:child];
        child = child->nextSibling;
    }
    
    LogIt(@"</%@>\n", [TBXML elementName:element]);
}
*/



static const int sCreditTypeCount = 29;

static NSString* sOrderedCreditTypes[sCreditTypeCount] =
{
    @"Code",
    @"Music",
    @"Graphics",
    @"BBS Graphics",
    @"Translation",
    @"Crack",
    @"Design",
    @"Idea",
    @"Text",
    @"Trainer",
    @"Charset",
    @"Original Supply",
    @"Linking",
    @"Concept",
    @"NTSC-Fix",
    @"Loader",
    @"Bug-Fix",
    @"Ripping",
    @"Docs",
    @"Interviewed",
    @"Import",
    @"Disk Cover",
    @"Help",
    @"Paper Art",
    @"PAL-Fix",
    @"Storyboard",
    @"Vote Sheet",
    @"Test",
    @"Sampling"
};



// ----------------------------------------------------------------------------
- (id) setFromXMLElement:(TBXMLElement*)releaseElement
// ----------------------------------------------------------------------------
{
    CSReleaseItem* release = self;
    
    release.identifier = [TBXML textForElement:[TBXML childElementNamed:@"ID" parentElement:releaseElement] withStringEncoding:NSUTF8StringEncoding];
    release.name = [TBXML textForElement:[TBXML childElementNamed:@"Name" parentElement:releaseElement] withStringEncoding:NSUTF8StringEncoding];
    release.name = [release.name stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
    release.name = [release.name stringByDecodingHTMLEntities];
    release.releaseType = [TBXML textForElement:[TBXML childElementNamed:@"Type" parentElement:releaseElement] withStringEncoding:NSUTF8StringEncoding];
    
    NSString* releaseDay = [TBXML textForElement:[TBXML childElementNamed:@"ReleaseDay" parentElement:releaseElement] withStringEncoding:NSUTF8StringEncoding];
    NSString* releaseMonth = [TBXML textForElement:[TBXML childElementNamed:@"ReleaseMonth" parentElement:releaseElement] withStringEncoding:NSUTF8StringEncoding];
    NSString* releaseYear = [TBXML textForElement:[TBXML childElementNamed:@"ReleaseYear" parentElement:releaseElement] withStringEncoding:NSUTF8StringEncoding];
    
    if (releaseDay != nil && releaseDay.length > 0)
        [_releaseDate setDay:[releaseDay integerValue]];
    else
        [_releaseDate setDay:0];
    
    if (releaseMonth != nil && releaseMonth.length > 0)
        [_releaseDate setMonth:[releaseMonth integerValue]];
    else
        [_releaseDate setMonth:0];
    
    if (releaseYear != nil && releaseYear.length > 0)
        [_releaseDate setYear:[releaseYear integerValue]];
    else
        [_releaseDate setYear:0];
    
    release.screenshot = [TBXML textForElement:[TBXML childElementNamed:@"ScreenShot" parentElement:releaseElement] withStringEncoding:NSUTF8StringEncoding];
    release.rating = [TBXML textForElement:[TBXML childElementNamed:@"Rating" parentElement:releaseElement] withStringEncoding:NSUTF8StringEncoding];

    TBXMLElement* releasedByElement = [TBXML childElementNamed:@"ReleasedBy" parentElement:releaseElement];
    if (releasedByElement != NULL)
    {
        TBXMLElement* groupElement = [TBXML childElementNamed:@"Group" parentElement:releasedByElement];
        while (groupElement != NULL)
        {
            CSGroupItem* group = [[CSGroupItem alloc] initWithXMLElement:groupElement];
            [release.releaseGroups addObject:group];
            
            groupElement = [TBXML nextSiblingNamed:@"Group" searchFromElement:groupElement];
        }

        TBXMLElement* handleElement = [TBXML childElementNamed:@"Handle" parentElement:releasedByElement];
        while (handleElement != NULL)
        {
            CSScenerItem* scener = [[CSScenerItem alloc] initWithXMLElement:handleElement];
            [release.releaseSceners addObject:scener];
            
            handleElement = [TBXML nextSiblingNamed:@"Handle" searchFromElement:handleElement];
        }
    }

    TBXMLElement* releasedAtElement = [TBXML childElementNamed:@"ReleasedAt" parentElement:releaseElement];
    if (releasedAtElement != NULL)
    {
        TBXMLElement* eventElement = [TBXML childElementNamed:@"Event" parentElement:releasedAtElement];
        if (eventElement != NULL)
        {
            NSString* eventId = [TBXML textForElement:[TBXML childElementNamed:@"ID" parentElement:eventElement] withStringEncoding:NSUTF8StringEncoding];
            
            release.releasedAtEvent = (CSEventItem*)[[CSDatabaseCache sharedInstance] itemForType:CSDB_EVENT withIdentifier:eventId];
            [release.releasedAtEvent setFromXMLElement:eventElement];
        }
        else
            release.releasedAtEvent = nil;
    }
    
    TBXMLElement* achievementElement = [TBXML childElementNamed:@"Achievement" parentElement:releaseElement];
    if (achievementElement != NULL)
    {
        TBXMLElement* compoElement = [TBXML childElementNamed:@"Compo" parentElement:achievementElement];
        TBXMLElement* placeElement = [TBXML childElementNamed:@"Place" parentElement:achievementElement];
        if (compoElement != NULL && placeElement != NULL && release.releasedAtEvent != nil)
        {
            NSString* compoName = [TBXML textForElement:compoElement withStringEncoding:NSUTF8StringEncoding];
            NSString* place = [TBXML textForElement:placeElement];
            NSString* lastDigit = [place substringWithRange:NSMakeRange(place.length - 1, 1)];
            int lastDigitValue = [lastDigit intValue];
            NSString* numberPostfix = @"th";
            switch (lastDigitValue)
            {
                case 1:
                    numberPostfix = @"st";
                    break;
                case 2:
                    numberPostfix = @"nd";
                    break;
                case 3:
                    numberPostfix = @"rd";
                    break;
            }
            release.achievement = [NSString stringWithFormat:@"%@%@ Place in %@ Compo", place, numberPostfix, compoName];
        }
        else
            release.achievement = nil;
    }
    
    TBXMLElement* creditsElement = [TBXML childElementNamed:@"Credits" parentElement:releaseElement];
    if (creditsElement != NULL)
    {
        TBXMLElement* creditElement = [TBXML childElementNamed:@"Credit" parentElement:creditsElement];
        while (creditElement != NULL)
        {
            CSCreditItem* creditItem = [[CSCreditItem alloc] initWithXMLElement:creditElement];
            NSMutableArray* creditsOfThisType = [release.creditsDictionary objectForKey:creditItem.type];
            if (creditsOfThisType == nil)
            {
                creditsOfThisType = [NSMutableArray array];
                [release.creditsDictionary setObject:creditsOfThisType forKey:creditItem.type];
                [release.creditTypes addObject:creditItem.type];
            }
            
            [creditsOfThisType addObject:creditItem];
            
            creditElement = [TBXML nextSiblingNamed:@"Credit" searchFromElement:creditElement];
        }

        NSArray* types = [release.creditsDictionary allKeys];
        for (NSString* type in types)
        {
            NSMutableArray* creditsOfThisType = [release.creditsDictionary objectForKey:type];
            if (creditsOfThisType != nil)
                [creditsOfThisType sortUsingSelector:@selector(compare:)];
        }
        
        NSMutableArray* sortedCreditTypes = [NSMutableArray arrayWithCapacity:release.creditTypes.count];
        
        for (int i=0; i<sCreditTypeCount; i++)
        {
            if ([release.creditTypes containsObject:sOrderedCreditTypes[i]])
                [sortedCreditTypes addObject:sOrderedCreditTypes[i]];
        }

        for (NSString* type in release.creditTypes)
        {
            BOOL found = NO;
            for (int i=0; i< sCreditTypeCount; i++)
            {
                if ([sOrderedCreditTypes[i] isEqualToString:type])
                {
                    found = YES;
                    break;
                }
            }
            
            if (!found)
                [sortedCreditTypes addObject:type];
        }
        
        release.creditTypes = sortedCreditTypes;
    }
    
    
    TBXMLElement* downloadLinksElement = [TBXML childElementNamed:@"DownloadLinks" parentElement:releaseElement];
    if (downloadLinksElement != NULL)
    {
        TBXMLElement* downloadLinkElement = [TBXML childElementNamed:@"DownloadLink" parentElement:downloadLinksElement];
        while (downloadLinkElement != NULL)
        {
            TBXMLElement* statusElement = [TBXML childElementNamed:@"Status" parentElement:downloadLinkElement];
            if (statusElement != NULL)
            {
                NSString* status = [TBXML textForElement:statusElement];
                if ([status isEqualToString:@"Ok"])
                {
                    TBXMLElement* linkElement = [TBXML childElementNamed:@"Link" parentElement:downloadLinkElement];
                    if (linkElement != NULL)
                    {
                        NSString* downloadURL = [TBXML textForElement:linkElement];
                        downloadURL = [downloadURL stringByAddingPercentEscapesUsingEncoding:NSASCIIStringEncoding];
                        [release.downloadURLs addObject:downloadURL];
                    }
                }
            }

            downloadLinkElement = [TBXML nextSiblingNamed:@"DownloadLink" searchFromElement:downloadLinkElement];
        }
    }    
    
    
    TBXMLElement* commentsElement = [TBXML childElementNamed:@"Comments" parentElement:releaseElement];
    if (commentsElement != NULL)
    {
        TBXMLElement* commentElement = commentsElement->firstChild;
        while (commentElement != NULL)
        {
            CSCommentItem* commentItem = [[CSCommentItem alloc] initWithXMLElement:commentElement];
            
            if ([[TBXML elementName:commentElement] isEqualToString:@"UserComment"])
                [release.comments addObject:commentItem];
            else if ([[TBXML elementName:commentElement] isEqualToString:@"Summary"])
                [release.summaries addObject:commentItem];
            else if ([[TBXML elementName:commentElement] isEqualToString:@"ProductionNote"])
                [release.productionNotes addObject:commentItem];
            else if ([[TBXML elementName:commentElement] isEqualToString:@"Goof"])
                [release.goofs addObject:commentItem];
            else if ([[TBXML elementName:commentElement] isEqualToString:@"HiddenPart"])
                [release.hiddenParts addObject:commentItem];
            else if ([[TBXML elementName:commentElement] isEqualToString:@"Trivia"])
                [release.trivias addObject:commentItem];
            
            commentElement = commentElement->nextSibling;
        }
    }
    
    return release;
}


// ----------------------------------------------------------------------------
- (NSComparisonResult) compareReleaseDates:(CSReleaseItem*)otherItem
// ----------------------------------------------------------------------------
{
    if (_releaseDate == nil || otherItem.releaseDate == nil)
        return NSOrderedSame;
    
    NSCalendar* calendar = [[NSCalendar alloc] initWithCalendarIdentifier:NSCalendarIdentifierGregorian];
    NSDate* date1 = [calendar dateFromComponents:_releaseDate];
    NSDate* date2 = [calendar dateFromComponents:otherItem.releaseDate];
    
    return [date2 compare:date1];
}



@end
