//
//  CSScenerItem.m
//  C64SceneBrowser
//
//  Created by Andreas Varga on 23.03.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "CSScenerItem.h"
#import "CSDatabaseCache.h"
#import "CSGroupItem.h"
#import "CSReleaseItem.h"
#import "CSEventItem.h"
#import "CSCreditItem.h"
#import "NSString+HTML.h"


@implementation CSScenerItem

@synthesize handle = _handle;
@synthesize alternativeHandle = _alternativeHandle;
@synthesize photo = _photo;
@synthesize country = _country;
@synthesize memberOf = _memberOf;
@synthesize released = _released;
@synthesize creditsDictionary = _creditsDictionary;
@synthesize sortedCreditReleases = _sortedCreditReleases;
@synthesize attendedEvents = _attendedEvents;
@synthesize organizedEvents = _organizedEvents;


// ----------------------------------------------------------------------------
- (id)init
// ----------------------------------------------------------------------------
{
    self = [super init];
    if (self)
    {
        _handle = nil;
        _alternativeHandle = nil;
        _photo = nil;
        _country = nil;
        _memberOf = [NSMutableArray array];
        _released = [NSMutableArray array];
        _creditsDictionary = [NSMutableDictionary dictionary];
        _sortedCreditReleases = [NSMutableArray array];
        _attendedEvents = [NSMutableArray array];
        _organizedEvents = [NSMutableArray array];
    }
    return self;
}


// ----------------------------------------------------------------------------
- (id) initWithScenerId:(NSString*)scenerId
// ----------------------------------------------------------------------------
{
    self = [super initWithIdentifier:scenerId];
    if (self)
    {
        _handle = nil;
        _alternativeHandle = nil;
        _photo = nil;
        _country = nil;
        _memberOf = [NSMutableArray array];
        _released = [NSMutableArray array];
        _creditsDictionary = [NSMutableDictionary dictionary];
        _sortedCreditReleases = [NSMutableArray array];
        _attendedEvents = [NSMutableArray array];
        _organizedEvents = [NSMutableArray array];
    }
    return self;
}


// ----------------------------------------------------------------------------
- (CSDatabaseItemType) type
// ----------------------------------------------------------------------------
{
    return CSDB_SCENER;
}


// ----------------------------------------------------------------------------
- (id) initWithXMLElement:(TBXMLElement*)handleElement
// ----------------------------------------------------------------------------
{
    self = [self init];
    if (self)
    {
        self = [self setFromXMLElement:handleElement];
    }
    return self;
}


// ----------------------------------------------------------------------------
- (id) setFromXMLElement:(TBXMLElement*)handleElement
// ----------------------------------------------------------------------------
{
    CSScenerItem* scener = self;
    
    TBXMLElement* scenerIdElement = [TBXML childElementNamed:@"ID" parentElement:handleElement];
    if (scenerIdElement != NULL)
        scener.identifier = [TBXML textForElement:scenerIdElement];
    
    if (scener.identifier != nil)
    {
        CSScenerItem* existing_scener = [[CSDatabaseCache sharedInstance].scenerDictionary objectForKey:scener.identifier];
        if (existing_scener != nil)
            scener = existing_scener;
        else
            [[CSDatabaseCache sharedInstance].scenerDictionary setObject:scener forKey:scener.identifier];
        
        TBXMLElement* handle2Element = [TBXML childElementNamed:@"Handle" parentElement:handleElement];
        if (handle2Element != NULL)
        {
            scener.handle = [TBXML textForElement:handle2Element withStringEncoding:NSUTF8StringEncoding];
            scener.handle = [scener.handle stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
            scener.handle = [scener.handle stringByDecodingHTMLEntities];
        }
        
        TBXMLElement* akaElement = [TBXML childElementNamed:@"AKA" parentElement:handleElement];
        if (akaElement != NULL)
        {
            scener.alternativeHandle = [TBXML textForElement:akaElement withStringEncoding:NSUTF8StringEncoding];
            scener.alternativeHandle = [scener.alternativeHandle stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
            scener.alternativeHandle = [scener.alternativeHandle stringByDecodingHTMLEntities];
        }
        
        TBXMLElement* scenerElement = [TBXML childElementNamed:@"Scener" parentElement:handleElement];
        if (scenerElement != NULL)
        {
            TBXMLElement* countryElement = [TBXML childElementNamed:@"Country" parentElement:scenerElement];
            if (countryElement != NULL)
                scener.country = [TBXML textForElement:countryElement withStringEncoding:NSUTF8StringEncoding];
        }
        
        TBXMLElement* photoElement = [TBXML childElementNamed:@"Photo" parentElement:handleElement];
        if (photoElement != NULL)
        {
            NSURL* url = [NSURL URLWithString:[TBXML textForElement:photoElement]];
            SDWebImageManager* manager = [SDWebImageManager sharedManager];
            [manager cancelForDelegate:scener];
            
            scener.photo = nil;
            
            if (url)
                [manager downloadWithURL:url delegate:scener options:(SDWebImageOptions)0];
        }
        else
        {
            NSInteger scener_id = [scener.identifier integerValue];
            NSInteger id_base_dir = scener_id < 1000 ? 1 : (1000*(scener_id/1000));
            NSURL* url = [NSURL URLWithString:[NSString stringWithFormat:@"http://csdb.dk/gfx/sceners/%ld/%ld.jpg", (long)id_base_dir, (long)scener_id]];
            SDWebImageManager* manager = [SDWebImageManager sharedManager];
            [manager cancelForDelegate:scener];
            
            scener.photo = nil;
            
            if (url)
                [manager downloadWithURL:url delegate:scener options:(SDWebImageOptions)0];
        }

        TBXMLElement* memberOfElement = [TBXML childElementNamed:@"MemberOf" parentElement:handleElement];
        while (memberOfElement != NULL)
        {
            CSMemberOfItem* memberOfItem = [[CSMemberOfItem alloc] initWithXMLElement:memberOfElement];
            [scener.memberOf addObject:memberOfItem];
            
            memberOfElement = [TBXML nextSiblingNamed:@"MemberOf" searchFromElement:memberOfElement];
        }
        
        TBXMLElement* releasedElement = [TBXML childElementNamed:@"Released" parentElement:handleElement];
        if (releasedElement != NULL)
        {
            TBXMLElement* releaseElement = [TBXML childElementNamed:@"Release" parentElement:releasedElement];
            while (releaseElement != NULL)
            {
                NSString* releaseId = [TBXML textForElement:[TBXML childElementNamed:@"ID" parentElement:releaseElement] withStringEncoding:NSUTF8StringEncoding];
                
                CSReleaseItem* releaseItem = (CSReleaseItem*)[[CSDatabaseCache sharedInstance] itemForType:CSDB_RELEASE withIdentifier:releaseId];
                if (releaseItem.cached_depth < 1)
                {
                    [releaseItem setFromXMLElement:releaseElement];
                    releaseItem.cached_depth = 1;
                }
                
                [scener.released addObject:releaseItem];
                
                releaseElement = [TBXML nextSiblingNamed:@"Release" searchFromElement:releaseElement];
            }
        }
        
        TBXMLElement* attendedElement = [TBXML childElementNamed:@"Attended" parentElement:handleElement];
        if (attendedElement != NULL)
        {
            TBXMLElement* eventElement = [TBXML childElementNamed:@"Event" parentElement:attendedElement];
            while (eventElement != NULL)
            {
                NSString* eventId = [TBXML textForElement:[TBXML childElementNamed:@"ID" parentElement:eventElement] withStringEncoding:NSUTF8StringEncoding];
                
                CSEventItem* eventItem = (CSEventItem*)[[CSDatabaseCache sharedInstance] itemForType:CSDB_EVENT withIdentifier:eventId];
                [eventItem setFromXMLElement:eventElement];
                [scener.attendedEvents addObject:eventItem];
                
                eventElement = [TBXML nextSiblingNamed:@"Event" searchFromElement:eventElement];
            }
        }

        TBXMLElement* organizedElement = [TBXML childElementNamed:@"Organized" parentElement:handleElement];
        if (organizedElement != NULL)
        {
            TBXMLElement* eventElement = [TBXML childElementNamed:@"Event" parentElement:organizedElement];
            while (eventElement != NULL)
            {
                NSString* eventId = [TBXML textForElement:[TBXML childElementNamed:@"ID" parentElement:eventElement] withStringEncoding:NSUTF8StringEncoding];
                
                CSEventItem* eventItem = (CSEventItem*)[[CSDatabaseCache sharedInstance] itemForType:CSDB_EVENT withIdentifier:eventId];
                [eventItem setFromXMLElement:eventElement];
                [scener.organizedEvents addObject:eventItem];
                
                eventElement = [TBXML nextSiblingNamed:@"Event" searchFromElement:eventElement];
            }
        }

        TBXMLElement* creditsElement = [TBXML childElementNamed:@"Credits" parentElement:handleElement];
        if (creditsElement != NULL)
        {
            TBXMLElement* creditElement = [TBXML childElementNamed:@"Credit" parentElement:creditsElement];
            while (creditElement != NULL)
            {
                CSCreditItem* creditItem = [[CSCreditItem alloc] initWithXMLElement:creditElement];
                NSString* releaseId = creditItem.releaseItem.identifier;
                if (![_sortedCreditReleases containsObject:creditItem.releaseItem])
                    [_sortedCreditReleases addObject:creditItem.releaseItem];
                
                NSMutableArray* creditsOfRelease = [scener.creditsDictionary objectForKey:releaseId];
                if (creditsOfRelease == nil)
                {
                    creditsOfRelease = [NSMutableArray array];
                    [scener.creditsDictionary setObject:creditsOfRelease forKey:releaseId];
                }
                
                [creditsOfRelease addObject:creditItem];
                
                creditElement = [TBXML nextSiblingNamed:@"Credit" searchFromElement:creditElement];
            }
        }

        // Sort by join date
        [scener.memberOf sortUsingSelector:@selector(compare:)];
        [scener.released sortUsingSelector:@selector(compareReleaseDates:)];
        [scener.attendedEvents sortUsingSelector:@selector(compareStartDates:)];
        [scener.organizedEvents sortUsingSelector:@selector(compareStartDates:)];
        [_sortedCreditReleases sortUsingSelector:@selector(compareReleaseDates:)];
    }
    
    return scener;
}


// ----------------------------------------------------------------------------
- (void) setFromXMLData:(NSData*)xmlData
// ----------------------------------------------------------------------------
{
    NSError* error;
    TBXML* tbxml = [TBXML tbxmlWithXMLData:xmlData error:&error];
    if (tbxml == nil || tbxml.rootXMLElement == NULL)
        return;

    TBXMLElement* handleElement = [TBXML childElementNamed:@"Handle" parentElement:tbxml.rootXMLElement];
    if (handleElement == NULL)
        return;
    [self setFromXMLElement:handleElement];
    
    [super setFromXMLData:xmlData];
}


// ----------------------------------------------------------------------------
- (void) cacheAndNotify:(id)owner withDepth:(NSUInteger)depth
// ----------------------------------------------------------------------------
{
    //NSLog(@"Caching scener %@, depth %d\n", self.identifier, depth);
    
    [super cacheAndNotify:owner withDepth:depth];
}


#pragma mark - SDWebImageManagerDelegate


// ----------------------------------------------------------------------------
- (void)webImageManager:(SDWebImageManager *)imageManager didFinishWithImage:(UIImage *)image
// ----------------------------------------------------------------------------
{
    _photo = image;
}


@end


#pragma mark -

@implementation CSMemberOfItem

@synthesize group = _group;
@synthesize professions = _professions;
@synthesize joinDate = _joinDate;
@synthesize leaveDate = _leaveDate;
@synthesize hasValidDates = _hasValidDates;
@synthesize status = _status;


// ----------------------------------------------------------------------------
- (id)init
// ----------------------------------------------------------------------------
{
    self = [super init];
    if (self)
    {
        _group = nil;
        _professions = [NSMutableArray array];
        _joinDate = nil;
        _leaveDate = nil;
        _status = nil;
    }
    return self;
}


// ----------------------------------------------------------------------------
- (id) initWithXMLElement:(TBXMLElement*)memberOfElement
// ----------------------------------------------------------------------------
{
    self = [self init];
    if (self)
    {
        self = [self setFromXMLElement:memberOfElement];
    }
    return self;
}


// ----------------------------------------------------------------------------
- (id) setFromXMLElement:(TBXMLElement*)memberOfElement
// ----------------------------------------------------------------------------
{
    TBXMLElement* groupElement = [TBXML childElementNamed:@"Group" parentElement:memberOfElement];
    if (groupElement != NULL)
    {
        CSGroupItem* groupItem = [[CSGroupItem alloc] initWithXMLElement:groupElement];
        _group = groupItem;
    }
    
    TBXMLElement* professionElement = [TBXML childElementNamed:@"Profession" parentElement:memberOfElement];
    while (professionElement != NULL)
    {
        NSString* profession = [TBXML textForElement:professionElement withStringEncoding:NSUTF8StringEncoding];
        [_professions addObject:profession];
        
        professionElement = [TBXML nextSiblingNamed:@"Profession" searchFromElement:professionElement];
    }

    _hasValidDates = NO;
    
    NSString* joinDay = nil;
    NSString* joinMonth = nil;
    NSString* joinYear = nil;
    
    TBXMLElement* joinDayElement = [TBXML childElementNamed:@"JoinDay" parentElement:memberOfElement];
    if (joinDayElement != NULL)
        joinDay = [TBXML textForElement:joinDayElement withStringEncoding:NSUTF8StringEncoding];
                               
    TBXMLElement* joinMonthElement = [TBXML childElementNamed:@"JoinMonth" parentElement:memberOfElement];
    if (joinMonthElement != NULL)
        joinMonth = [TBXML textForElement:joinMonthElement withStringEncoding:NSUTF8StringEncoding];

    TBXMLElement* joinYearElement = [TBXML childElementNamed:@"JoinYear" parentElement:memberOfElement];
    if (joinYearElement != NULL)
        joinYear = [TBXML textForElement:joinYearElement withStringEncoding:NSUTF8StringEncoding];
    
    _joinDate = [[NSDateComponents alloc] init];
    
    if (joinDay != nil)
        [_joinDate setDay:[joinDay integerValue]];
    else
        [_joinDate setDay:1];
    
    if (joinMonth != nil)
        [_joinDate setMonth:[joinMonth integerValue]];
    else
        [_joinDate setMonth:1];
    
    if (joinYear != nil)
    {
        [_joinDate setYear:[joinYear integerValue]];
        _hasValidDates = YES;
    }
    else
        [_joinDate setYear:0];

    NSString* leaveDay = nil;
    NSString* leaveMonth = nil;
    NSString* leaveYear = nil;
    
    TBXMLElement* leaveDayElement = [TBXML childElementNamed:@"LeaveDay" parentElement:memberOfElement];
    if (leaveDayElement != NULL)
        leaveDay = [TBXML textForElement:leaveDayElement withStringEncoding:NSUTF8StringEncoding];
    
    TBXMLElement* leaveMonthElement = [TBXML childElementNamed:@"LeaveMonth" parentElement:memberOfElement];
    if (leaveMonthElement != NULL)
        leaveMonth = [TBXML textForElement:leaveMonthElement withStringEncoding:NSUTF8StringEncoding];
    
    TBXMLElement* leaveYearElement = [TBXML childElementNamed:@"LeaveYear" parentElement:memberOfElement];
    if (leaveYearElement != NULL)
        leaveYear = [TBXML textForElement:leaveYearElement withStringEncoding:NSUTF8StringEncoding];
    
    _leaveDate = [[NSDateComponents alloc] init];
    
    if (leaveDay != nil)
        [_leaveDate setDay:[leaveDay integerValue]];
    else
        [_leaveDate setDay:1];
    
    if (leaveMonth != nil)
        [_leaveDate setMonth:[leaveMonth integerValue]];
    else
        [_leaveDate setMonth:1];
    
    if (leaveYear != nil)
    {
        [_leaveDate setYear:[leaveYear integerValue]];
        _hasValidDates = YES;
    }
    else
        [_leaveDate setYear:0];
    
    TBXMLElement* statusElement = [TBXML childElementNamed:@"Status" parentElement:memberOfElement];
    if (statusElement != NULL)
        _status = [TBXML textForElement:statusElement withStringEncoding:NSUTF8StringEncoding];
    
    return self;
}



// ----------------------------------------------------------------------------
- (NSComparisonResult) compare:(CSMemberOfItem*)otherItem;
// ----------------------------------------------------------------------------
{
    if (_joinDate == nil || otherItem.joinDate == nil)
        return NSOrderedSame;

    NSCalendar* calendar = [[NSCalendar alloc] initWithCalendarIdentifier:NSCalendarIdentifierGregorian];
    NSDate* date1 = [calendar dateFromComponents:_joinDate];
    NSDate* date2 = [calendar dateFromComponents:otherItem.joinDate];
    
    //NSLog(@"compare %@ with %@", date1, date2);
    
    return [date1 compare:date2];
}


@end


