//
//  CreditsViewController.m
//  C64SceneBrowser
//
//  Created by Andreas Varga on 24.03.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "CreditsViewController.h"
#import "MasterViewController.h"
#import "StartPageViewController.h"
#import "CSCreditItem.h"
#import "CSScenerItem.h"
#import "CSReleaseItem.h"
#import "ScenerViewController.h"
#import "ReleaseViewController.h"
#import "AppDelegate.h"


@implementation CreditsViewController

@synthesize releaseItem = _releaseItem;
@synthesize selectedItem = _selectedItem;


// ----------------------------------------------------------------------------
- (id) initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
// ----------------------------------------------------------------------------
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self)
    {
        _selectedItem = nil;
    }
    return self;
}


// ----------------------------------------------------------------------------
- (void) viewDidLoad
// ----------------------------------------------------------------------------
{
    [super viewDidLoad];
    
	// Do any additional setup after loading the view.
    //self.navigationItem.hidesBackButton = YES;
}


// ----------------------------------------------------------------------------
- (void) viewDidUnload
// ----------------------------------------------------------------------------
{

    [super viewDidUnload];
    // Release any retained subviews of the main view.
}


// ----------------------------------------------------------------------------
- (BOOL) shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
// ----------------------------------------------------------------------------
{
    return YES;
}


// ----------------------------------------------------------------------------
- (void) updateView
// ----------------------------------------------------------------------------
{
    [_creditsTableView reloadData];
}


// ----------------------------------------------------------------------------
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
// ----------------------------------------------------------------------------
{
    if ([[segue identifier] isEqualToString:@"ShowScenerDetail"])
    {
        ScenerViewController* scenerViewController = [segue destinationViewController];
        scenerViewController.scenerItem = _selectedItem;
    }
}


#pragma mark TableView delegate and datasource

// ----------------------------------------------------------------------------
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
// ----------------------------------------------------------------------------
{
    return _releaseItem.creditTypes.count;
}


// ----------------------------------------------------------------------------
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
// ----------------------------------------------------------------------------
{
    if (_releaseItem.creditTypes == nil || section >= _releaseItem.creditTypes.count)
        return 0;
    
    NSString* type = [_releaseItem.creditTypes objectAtIndex:section];
    NSMutableArray* creditsOfThisType = [_releaseItem.creditsDictionary objectForKey:type];
    if (creditsOfThisType == nil)
        return 0;
    
    return creditsOfThisType.count;
}


// ----------------------------------------------------------------------------
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
// ----------------------------------------------------------------------------
{
    CreditItemCell* cell = [tableView dequeueReusableCellWithIdentifier:@"CreditItemCell"];

    if (_releaseItem.creditTypes == nil || indexPath.section >= _releaseItem.creditTypes.count)
        return 0;

    NSString* type = [_releaseItem.creditTypes objectAtIndex:indexPath.section];
    NSMutableArray* creditsOfThisType = [_releaseItem.creditsDictionary objectForKey:type];

    CSCreditItem* credit = [creditsOfThisType objectAtIndex:indexPath.row];
    if (credit != nil)
    {
        cell.handleLabel.text = credit.scener.handle;
        cell.photoImageView.image = credit.scener.photo;
    }

    cell.contentView.backgroundColor = ((indexPath.row & 1) == 0) ? [UIColor blackColor] : [UIColor colorWithWhite:0.1f alpha:1.0f];

    return cell;
}


// ----------------------------------------------------------------------------
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
// ----------------------------------------------------------------------------
{
    return NO;
}


// ----------------------------------------------------------------------------
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
// ----------------------------------------------------------------------------
{
    if (_releaseItem.creditTypes == nil || section >= _releaseItem.creditTypes.count)
        return 0;
    
    NSString* type = [_releaseItem.creditTypes objectAtIndex:section];

    return type;
}


// ----------------------------------------------------------------------------
- (UIView*) tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
// ----------------------------------------------------------------------------
{
    NSString *sectionTitle = [self tableView:tableView titleForHeaderInSection:section];
    if (sectionTitle == nil)
        return nil;
    
    CGFloat width = tableView.frame.size.width;
    
    UIView* customView = [[UIView alloc] init];
    customView.backgroundColor = [UIColor lightGrayColor]; //[UIColor colorWithPatternImage:[UIImage imageNamed:@"section_header"]];
    
    CGFloat left_inset = 15.0f;
    UILabel* headerLabel = [[UILabel alloc] initWithFrame:CGRectMake(left_inset, -23.0f, width - left_inset, 46.0f)];
    headerLabel.backgroundColor = [UIColor clearColor];
    headerLabel.autoresizingMask = UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;
    headerLabel.opaque = NO;
    headerLabel.textColor = [UIColor darkGrayColor];
    headerLabel.font = [UIFont boldSystemFontOfSize:14.0f];
    headerLabel.textAlignment = NSTextAlignmentLeft;
    headerLabel.text = sectionTitle;
    [customView addSubview:headerLabel];
    return customView;
}



// ----------------------------------------------------------------------------
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
// ----------------------------------------------------------------------------
{
    NSString* type = [_releaseItem.creditTypes objectAtIndex:indexPath.section];
    NSMutableArray* creditsOfThisType = [_releaseItem.creditsDictionary objectForKey:type];
    
    CSCreditItem* credit = [creditsOfThisType objectAtIndex:indexPath.row];
    if (credit != nil && (INTERFACE_IS_PHONE || credit.scener != _selectedItem))
    {
        MasterViewController* masterViewController = (MasterViewController*) self.navigationController.topViewController;
        ScenerViewController* scenerViewController = (ScenerViewController *)[[masterViewController.splitViewController.viewControllers lastObject] topViewController];
        
        if ([scenerViewController isKindOfClass:[ScenerViewController class]])
        {
            if (scenerViewController.isAppearing)
                return;
        }
        
        _selectedItem = credit.scener;
        [self performSegueWithIdentifier:@"ShowScenerDetail" sender:self];
        
        AppDelegate* appDelegate = (AppDelegate*)[[UIApplication sharedApplication] delegate];
        if (appDelegate.masterPopoverController != nil)
            [appDelegate.masterPopoverController dismissPopoverAnimated:YES];
    }
}


@end



@implementation CreditItemCell

@synthesize handleLabel = _handleLabel;
@synthesize photoImageView = _photoImageView;

@end


