//
//  EmulationViewController.h
//  C64SceneBrowser
//
//  Created by Andreas Varga on 03.04.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import <UIKit/UIKit.h>
#import <MediaPlayer/MediaPlayer.h>
#import "libx64/vice/src/arch/unix/ios/cocoa_touch/viceapplicationprotocol.h"


@class VICEGLView;
@class SSLoadingView;
@class EmulationKeyboardResponder;
@class JoystickView;
@class FireButtonView;
@class ReleaseViewController;
@class OscilloscopeView;
@class OHAttributedLabel;
@class GCController;


enum EJoystickMode
{
    JOYSTICK_DISABLED = 0,
    JOYSTICK_PORT1,
    JOYSTICK_PORT2,
    
    JOYSTICK_MODE_COUNT
};


@interface CSVolumeView : MPVolumeView

@end



@interface EmulationViewController : UIViewController <VICEApplicationProtocol, UIToolbarDelegate>
{
    IBOutlet VICEGLView*            _viceView;
    IBOutlet UIToolbar*             _toolBar;
    IBOutlet UIToolbar*             _bottomToolBar;
    IBOutlet UIView*                _portraitBottomToolBarView;
    IBOutlet UILabel*               _titleLabel;
    IBOutlet UIButton*              _warpButton;
    IBOutlet UIButton*              _warpButton2;
    IBOutlet UIButton*              _playPauseButton;
    IBOutlet UIButton*              _playPauseButton2;
    IBOutlet JoystickView*          _joystickView;
    IBOutlet FireButtonView*        _fireButtonView;
    IBOutlet UIBarButtonItem*       _joystickModeButton;
    IBOutlet UIBarButtonItem*       _joystickModeButton2;
    IBOutlet UIBarButtonItem*       _keyboardButton;
    IBOutlet UIBarButtonItem*       _keyboardButton2;
    IBOutlet UIBarButtonItem*       _settingsButton;
    IBOutlet UIBarButtonItem*       _settingsButton2;
    IBOutlet CSVolumeView*          _volumeView;
    IBOutlet CSVolumeView*          _volumeView2;
    IBOutlet UIImageView*           _driveLedView;
    IBOutlet UIView*                _emulationBackgroundView;
    IBOutlet OscilloscopeView*      _oscilloscopeView;
    IBOutlet UIView*                _sidInfoView;
    IBOutlet OHAttributedLabel*     _sidTitleLabel;
    IBOutlet OHAttributedLabel*     _sidAuthorLabel;
    IBOutlet OHAttributedLabel*     _sidReleasedLabel;
    IBOutlet OHAttributedLabel*     _sidSongsLabel;
    
    NSArray*                        _arguments;
    BOOL                            _canTerminate;
    NSMutableArray*                 _supportedFiles;
    NSString*                       _dataFilePath;
    
    BOOL                            _emulationRunning;
    BOOL                            _warpEnabled;
    CGFloat                         _keyboardOffset;
    BOOL                            _viceViewScaled;

    NSTimer*                        _controlFadeTimer;
    BOOL                            _controlsVisible;
    
    NSMutableData*                  _downloadData;
    NSURLConnection*                _downloadUrlConnection;
    NSString*                       _downloadFilename;
    
    SSLoadingView*                  _loadingView;
    
    UIPopoverController*            _settingsPopOver;
    
    UIColor*                        _activeLedColor;
    UIColor*                        _inactiveLedColor;
    
    NSTimer*                        _oscilloscopeTimer;
    GCController*                   _gameController;
}

@property (nonatomic) NSString* dataFileURLString;
@property (nonatomic) NSString* releaseId;
@property (nonatomic) EJoystickMode joystickMode;
@property (nonatomic) ReleaseViewController* releaseViewController;
@property (nonatomic) BOOL keyboardVisible;


- (IBAction) clickDoneButton:(id)sender;
- (IBAction) clickSettingsButton:(id)sender;
- (IBAction) clickKeyboardButton:(id)sender;
- (IBAction) clickJoystickButton:(id)sender;

- (IBAction) clickPlayPauseButton:(id)sender;
- (IBAction) clickRestartButton:(id)sender;
- (IBAction) clickWarpButton:(id)sender;

- (BOOL) controlsVisible;
- (void) controlsFadeIn;
- (void) controlsFadeOut;
- (BOOL) isRunning;

- (void) shrinkOrGrowViceView;
- (void) setIsInBackground:(BOOL)isInBackground;

- (void) setupGameController;

@end


@interface EmulationBackgroundView : UIView
{
    IBOutlet EmulationViewController*    _emulationViewController;
    BOOL                                 _keyboardWasVisible;
}


@end


@interface EmulationKeyboardResponder : UIView <UIKeyInput>
{
    
}

@end


@interface JoystickView : UIImageView
{
    IBOutlet EmulationViewController*    _emulationViewController;

    NSMutableArray*     _upButtonTouches;
    NSMutableArray*     _downButtonTouches;
    NSMutableArray*     _leftButtonTouches;
    NSMutableArray*     _rightButtonTouches;
}

@end


@interface FireButtonView : UIImageView
{
    IBOutlet EmulationViewController*    _emulationViewController;

    NSMutableArray*     _fireButtonTouches;
}

@end

