//
//  VICESettingsViewController.m
//  C64SceneBrowser
//
//  Created by Andreas Varga on 05.04.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "EmulatorSettingsViewController.h"
#import "libx64/vice/src/arch/unix/ios/cocoa_touch/vicemachine.h"
#import "libx64/vice/src/c64/c64model.h"
#import "DiskImageViewController.h"


@implementation EmulatorSettingsViewController


// ----------------------------------------------------------------------------
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
// ----------------------------------------------------------------------------
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}


// ----------------------------------------------------------------------------
- (void)viewDidLoad
// ----------------------------------------------------------------------------
{
    [super viewDidLoad];
}


// ----------------------------------------------------------------------------
- (void)viewDidUnload
// ----------------------------------------------------------------------------
{
    [super viewDidUnload];
    
    // Release any retained subviews of the main view.
}


// ----------------------------------------------------------------------------
- (void)viewWillAppear:(BOOL)animated
// ----------------------------------------------------------------------------
{
    [super viewWillAppear:animated];
    
    [self.tableView reloadData];
}


// ----------------------------------------------------------------------------
- (void)viewWillDisappear:(BOOL)animated
// ----------------------------------------------------------------------------
{
    [super viewWillDisappear:animated];
    
    [[theVICEMachine machineController] saveResources:nil];
}


// ----------------------------------------------------------------------------
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
// ----------------------------------------------------------------------------
{
	return YES;
}


#pragma mark TableView delegate and datasource

// ----------------------------------------------------------------------------
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
// ----------------------------------------------------------------------------
{
    return 4;
}


// ----------------------------------------------------------------------------
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
// ----------------------------------------------------------------------------
{
    switch (section)
    {
        case 0:
            return 1;
        case 1:
            return 1;
        case 2:
            return 2;
        case 3:
            return 1;
    }
    
    return 0;
}    


static NSString* sDetailStringForModel[] =
{
    @"PAL C64",
    @"PAL C64C",
    @"PAL C64 (old)",
    @"NTSC C64",
    @"NTSC C64C",
    @"NTSC C64 (old)",
    @"Drean C64"
};


// ----------------------------------------------------------------------------
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
// ----------------------------------------------------------------------------
{
    UITableViewCell* cell = nil;
    
    switch (indexPath.section)
    {
        case 0:
        {
            cell = [tableView dequeueReusableCellWithIdentifier:@"C64ModelCell"];
            int model = [[theVICEMachine machineController] getModel];
            if (model == C64MODEL_UNKNOWN)
                cell.detailTextLabel.text = @"Unknown";
            else
                cell.detailTextLabel.text = sDetailStringForModel[model];
            break;
        }
        case 1:
            cell = [tableView dequeueReusableCellWithIdentifier:@"DiskTapeCell"];
            cell.detailTextLabel.text = [theVICEMachine driveImageName];
            break;
        case 2:
            
            switch (indexPath.row) 
            {
                case 0:
                    cell = [tableView dequeueReusableCellWithIdentifier:@"EmulatorScreenSettingsCell"];
                    ((EmulatorScreenSettingsCell*)cell).saturationSlider.value = [[[theVICEMachine machineController] getIntResource:@"VICIIColorSaturation"] floatValue];
                    ((EmulatorScreenSettingsCell*)cell).contrastSlider.value = [[[theVICEMachine machineController] getIntResource:@"VICIIColorContrast"] floatValue];
                    ((EmulatorScreenSettingsCell*)cell).brightnessSlider.value = [[[theVICEMachine machineController] getIntResource:@"VICIIColorBrightness"] floatValue];
                    ((EmulatorScreenSettingsCell*)cell).gammaSlider.value = [[[theVICEMachine machineController] getIntResource:@"VICIIColorGamma"] floatValue];
                    ((EmulatorScreenSettingsCell*)cell).blurSlider.value = [[[theVICEMachine machineController] getIntResource:@"VICIIPALBlur"] floatValue];
                    ((EmulatorScreenSettingsCell*)cell).scanlineShadeSlider.value = [[[theVICEMachine machineController] getIntResource:@"VICIIPALScanLineShade"] floatValue];
                    ((EmulatorScreenSettingsCell*)cell).oddLinePhaseSlider.value = [[[theVICEMachine machineController] getIntResource:@"VICIIPALOddLinePhase"] floatValue];
                    ((EmulatorScreenSettingsCell*)cell).oddLineOffsetSlider.value = [[[theVICEMachine machineController] getIntResource:@"VICIIPALOddLineOffset"] floatValue];
                    break;
                case 1:
                    cell = [tableView dequeueReusableCellWithIdentifier:@"ResetScreenCell"];
                    break;
            }
            break;
        case 3:
            cell = [tableView dequeueReusableCellWithIdentifier:@"ResetMachineCell"];
            break;
    }
    
    return cell;
}


// ----------------------------------------------------------------------------
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
// ----------------------------------------------------------------------------
{
    return NO;
}


// ----------------------------------------------------------------------------
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
// ----------------------------------------------------------------------------
{
    return @"";
}


// ----------------------------------------------------------------------------
- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
// ----------------------------------------------------------------------------
{
    switch (indexPath.section)
    {
        case 0:
            return 45.0f;
            break;
        case 1:
            return 45.0f;
            break;
        case 2:
            switch (indexPath.row) 
            {
                case 0:
                    return 265.0f; //413.0f;
                    break;
                case 1:
                    return 45.0f;
                    break;
            }
            break;
        case 3:
            return 45.0f;
            break;
    }
     
    return 45.0f;
}


// ----------------------------------------------------------------------------
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
// ----------------------------------------------------------------------------
{
    //NSLog(@"Selected row %d of section %d", indexPath.row, indexPath.section);
}


// ----------------------------------------------------------------------------
- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath
// ----------------------------------------------------------------------------
{
    if (indexPath.section == 2)
        return nil;
    
    return indexPath;
}


// ----------------------------------------------------------------------------
- (IBAction)changeSaturation:(id)sender
// ----------------------------------------------------------------------------
{
    float value = [(UISlider*)sender value];
    VICEMachineResourceKeyValue* keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIColorSaturation" andValue:[NSNumber numberWithFloat:value]];
    [theVICEMachine performSelector:@selector(setResourceOnceAfterDelay:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:NO];
}


// ----------------------------------------------------------------------------
- (IBAction)changeContrast:(id)sender
// ----------------------------------------------------------------------------
{
    float value = [(UISlider*)sender value];
    VICEMachineResourceKeyValue* keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIColorContrast" andValue:[NSNumber numberWithFloat:value]];
    [theVICEMachine performSelector:@selector(setResourceOnceAfterDelay:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:NO];
}


// ----------------------------------------------------------------------------
- (IBAction)changeBrightness:(id)sender
// ----------------------------------------------------------------------------
{
    float value = [(UISlider*)sender value];
    VICEMachineResourceKeyValue* keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIColorBrightness" andValue:[NSNumber numberWithFloat:value]];
    [theVICEMachine performSelector:@selector(setResourceOnceAfterDelay:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:NO];
}


// ----------------------------------------------------------------------------
- (IBAction)changeGamma:(id)sender
// ----------------------------------------------------------------------------
{
    float value = [(UISlider*)sender value];
    VICEMachineResourceKeyValue* keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIColorGamma" andValue:[NSNumber numberWithFloat:value]];
    [theVICEMachine performSelector:@selector(setResourceOnceAfterDelay:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:NO];
}


// ----------------------------------------------------------------------------
- (IBAction)changeBlur:(id)sender
// ----------------------------------------------------------------------------
{
    float value = [(UISlider*)sender value];
    VICEMachineResourceKeyValue* keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIPALBlur" andValue:[NSNumber numberWithFloat:value]];
    [theVICEMachine performSelector:@selector(setResourceOnceAfterDelay:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:NO];
}


// ----------------------------------------------------------------------------
- (IBAction)changeScanlineShade:(id)sender
// ----------------------------------------------------------------------------
{
    float value = [(UISlider*)sender value];
    VICEMachineResourceKeyValue* keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIPALScanLineShade" andValue:[NSNumber numberWithFloat:value]];
    [theVICEMachine performSelector:@selector(setResourceOnceAfterDelay:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:NO];
}


// ----------------------------------------------------------------------------
- (IBAction)changeOddLinePhase:(id)sender
// ----------------------------------------------------------------------------
{
    float value = [(UISlider*)sender value];
    VICEMachineResourceKeyValue* keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIPALOddLinePhase" andValue:[NSNumber numberWithFloat:value]];
    [theVICEMachine performSelector:@selector(setResourceOnceAfterDelay:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:NO];
}


// ----------------------------------------------------------------------------
- (IBAction)changeOddLineOffset:(id)sender
// ----------------------------------------------------------------------------
{
    float value = [(UISlider*)sender value];
    VICEMachineResourceKeyValue* keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIPALOddLineOffset" andValue:[NSNumber numberWithFloat:value]];
    [theVICEMachine performSelector:@selector(setResourceOnceAfterDelay:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:NO];
}


// ----------------------------------------------------------------------------
- (IBAction)resetScreenToDefaults:(id)sender
// ----------------------------------------------------------------------------
{
    VICEMachineResourceKeyValue* keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIColorSaturation" andValue:[NSNumber numberWithFloat:1000.0f]];
    [theVICEMachine performSelector:@selector(setResource:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:YES];
    
    keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIColorContrast" andValue:[NSNumber numberWithFloat:1000.0f]];
    [theVICEMachine performSelector:@selector(setResource:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:YES];
    
    keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIColorBrightness" andValue:[NSNumber numberWithFloat:1000.0f]];
    [theVICEMachine performSelector:@selector(setResource:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:YES];
    
    keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIColorGamma" andValue:[NSNumber numberWithFloat:2200.0f]];
    [theVICEMachine performSelector:@selector(setResource:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:YES];

    keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIPALBlur" andValue:[NSNumber numberWithFloat:0.0f]];
    [theVICEMachine performSelector:@selector(setResource:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:YES];

    keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIPALScanLineShade" andValue:[NSNumber numberWithFloat:800.0f]];
    [theVICEMachine performSelector:@selector(setResource:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:YES];
    
    keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIPALOddLinePhase" andValue:[NSNumber numberWithFloat:1000.0f]];
    [theVICEMachine performSelector:@selector(setResource:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:YES];
    
    keyValue = [[VICEMachineResourceKeyValue alloc] initWithResource:@"VICIIPALOddLineOffset" andValue:[NSNumber numberWithFloat:1000.0f]];
    [theVICEMachine performSelector:@selector(setResource:) onThread:[theVICEMachine machineThread] withObject:keyValue waitUntilDone:YES];
    
    NSArray* indexPaths = [NSArray arrayWithObject:[NSIndexPath indexPathForRow:0 inSection:2]];
    [self.tableView reloadRowsAtIndexPaths:indexPaths withRowAnimation:UITableViewRowAnimationNone];
}



// ----------------------------------------------------------------------------
- (IBAction) resetMachine:(id)sender
// ----------------------------------------------------------------------------
{
    UIAlertView* alertView = [[UIAlertView alloc] initWithTitle:@"WARNING" message:@"Do you really want to reset?" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"Reset", nil];
    [alertView show];
}


// ----------------------------------------------------------------------------
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
// ----------------------------------------------------------------------------
{
    if (buttonIndex > 0)
    {
        [[theVICEMachine machineController] resetMachine:YES];
    }
}


// ----------------------------------------------------------------------------
- (IBAction)dismiss:(id)sender
// ----------------------------------------------------------------------------

{
    [self dismissViewControllerAnimated:YES completion:NULL];
}



@end


@implementation EmulatorScreenSettingsCell

@synthesize saturationSlider;
@synthesize brightnessSlider;
@synthesize contrastSlider;
@synthesize gammaSlider;
@synthesize blurSlider;
@synthesize scanlineShadeSlider;
@synthesize oddLinePhaseSlider;
@synthesize oddLineOffsetSlider;


@end
