//
//  MembershipView.m
//  C64SceneBrowser
//
//  Created by Andreas Varga on 26.05.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "MembershipView.h"
#import "CSScenerItem.h"
#import "CSGroupItem.h"
#import <QuartzCore/QuartzCore.h>


@implementation MembershipView

@synthesize scener = _scener;


static const float sLegendHeight = 32.0f;
static const float sLegendWidth = 510.0f;


// ----------------------------------------------------------------------------
- (id)initWithFrame:(CGRect)frame
// ----------------------------------------------------------------------------
{
    self = [super initWithFrame:frame];
    if (self)
    {
//        CATiledLayer* tiled_layer = (CATiledLayer*) self.layer;
//        tiled_layer.levelsOfDetail = 5;
//        tiled_layer.levelsOfDetailBias = 2;
//        self.opaque = YES;
        
        _scener = nil;
    }
    return self;
}


//// ----------------------------------------------------------------------------
//+ (Class) layerClass
//// ----------------------------------------------------------------------------
//{
//    return [CATiledLayer class];
//}


// ----------------------------------------------------------------------------
- (void) setScener:(CSScenerItem *)scener
// ----------------------------------------------------------------------------
{
    _scener = scener;
    
    _earliestYear = INT_MAX;
    _latestYear = 0.0f;
    
    _hasAtleastOneDate = NO;
    for (CSMemberOfItem* member in _scener.memberOf)
    {
        if (member.hasValidDates)
            _hasAtleastOneDate = YES;
    }
    
    NSInteger currentYear = [[[NSCalendar currentCalendar] components:NSCalendarUnitYear fromDate:[NSDate date]] year];
    
    for (CSMemberOfItem* member in scener.memberOf)
    {
        //NSLog(@"Group: %@ (%@ -> %@)", member.group.name, member.joinDate, member.leaveDate);
        
        NSInteger joinYear = member.joinDate.year;
        if (joinYear != 0)
        {
            if (joinYear < _earliestYear)
                _earliestYear = joinYear;
            
            if (joinYear > _latestYear)
                _latestYear = joinYear;
        }
        
        if (member.leaveDate != nil)
        {
            NSInteger leaveYear = member.leaveDate.year;
            if (leaveYear == 0)
                leaveYear = currentYear;
                
            if (leaveYear < _earliestYear)
                _earliestYear = leaveYear;
            
            if (leaveYear > _latestYear)
                _latestYear = leaveYear;
        }
    }
    
    //NSLog(@"Range: %d to %d\n", _earliestYear, _latestYear);
}


// ----------------------------------------------------------------------------
- (float) desiredWidth
// ----------------------------------------------------------------------------
{
    [self drawMembership:NULL];

    float superViewWidth = [self.superview bounds].size.width;
    float superViewHeight = [self.superview bounds].size.height;
    float width = MAX(superViewWidth, superViewHeight);

    return INTERFACE_IS_PHONE ? MAX(width, _maxWidth) : _maxWidth;
}


// ----------------------------------------------------------------------------
- (float) desiredHeight
// ----------------------------------------------------------------------------
{
    [self drawMembership:NULL];
    
    float superViewWidth = [self.superview bounds].size.width;
    float superViewHeight = [self.superview bounds].size.height;
    float height = MIN(superViewWidth, superViewHeight);

    float minHeightForView = INTERFACE_IS_PHONE ? MAX(height - 44.0f, _maxHeight) : MAX(160.0f + sLegendHeight, _maxHeight + 34.0f);
    
    return minHeightForView;
}


// ----------------------------------------------------------------------------
- (void) drawRect:(CGRect)rect
// ----------------------------------------------------------------------------
{
    CGContextRef context = UIGraphicsGetCurrentContext();
    [self drawMembership:context];    
}


//// ----------------------------------------------------------------------------
//- (void) drawLayer:(CALayer*)layer inContext:(CGContextRef)context
//// ----------------------------------------------------------------------------
//{
//    [self drawMembership:context];
//}



// ----------------------------------------------------------------------------
static void sDrawGradientRect(CGContextRef context, CGRect rect, CGGradientRef gradient)
// ----------------------------------------------------------------------------
{
    CGPoint startPoint = CGPointMake(CGRectGetMidX(rect), CGRectGetMinY(rect));
    CGPoint endPoint = CGPointMake(CGRectGetMidX(rect), CGRectGetMaxY(rect));
    
    CGContextSaveGState(context);
    CGContextAddRect(context, rect);
    CGContextClip(context);
    CGContextDrawLinearGradient(context, gradient, startPoint, endPoint, 0);
    CGContextRestoreGState(context);
}



// ----------------------------------------------------------------------------
- (void) drawMembership:(CGContextRef)context
// ----------------------------------------------------------------------------
{
    CGRect view_rect = self.bounds;
    
    if (context != NULL)
    {
        CGContextSetRGBFillColor(context, 0.85f, 0.85f, 0.85f, 1.0f);
        CGContextFillRect(context, view_rect);
    }

    _maxHeight = 0.0f;
    _maxWidth = 0.0f;
    
    if (_scener == nil)
        return;
    
    float left_margin = 16.5f;
    float top_margin = 4.5f;
    float baseline_y = view_rect.size.height - sLegendHeight - 26.5f;
    
    if (context != NULL)
    {
        CGContextSetLineWidth(context, 1.0f);
        CGContextSelectFont(context, "Helvetica Neue", 12.0f, kCGEncodingMacRoman);
        CGContextSetRGBStrokeColor(context, 0.0f, 0.0f, 0.0f, 1.0f);
        CGContextSetRGBFillColor(context, 0.0f, 0.0f, 0.0f, 1.0f);
        CGContextSetTextMatrix(context, CGAffineTransformMakeScale(1.0f, -1.0f));
        CGContextSetTextDrawingMode(context, kCGTextFill);
    }
        
    // Draw timeline
    
    if (_hasAtleastOneDate && context != NULL)
    {
        float xpos = 0.0f;
        for (NSInteger year = _earliestYear; year <= _latestYear; year++)
        {
            CGContextSetRGBStrokeColor(context, 0.7f, 0.7f, 0.7f, 1.0f);
            
            // Year label
            char year_string[5];
            snprintf(year_string, 5, "%ld", (long)year);
            CGContextShowTextAtPoint(context, 3.0f + xpos, view_rect.size.height - sLegendHeight - 12.0f, year_string, 4);
            
            // Line for each year
            CGContextSetRGBStrokeColor(context, 0.3f, 0.3f, 0.3f, 1.0f);
            CGContextBeginPath(context);
            CGContextMoveToPoint(context, left_margin + xpos, baseline_y + 3.0f);
            CGContextAddLineToPoint(context, left_margin + xpos, 0.0f);
            CGContextStrokePath(context);
            
            CGContextSetRGBStrokeColor(context, 0.6f, 0.6f, 0.6f, 1.0f);
            
            // Lines for the months in the year
            for (int i=1; i<12; i++)
            {
                CGContextBeginPath(context);
                CGContextMoveToPoint(context, left_margin + xpos + float(i)*8.0f, baseline_y + 2.0f);
                CGContextAddLineToPoint(context, left_margin + xpos + float(i)*8.0f, 0.0f);
                CGContextStrokePath(context);
            }
            
            xpos += 96.0f;
        }
        
        // Horizontal base line
        CGContextSetRGBStrokeColor(context, 0.7f, 0.7f, 0.7f, 1.0f);
        CGContextBeginPath(context);
        CGContextMoveToPoint(context, left_margin, baseline_y);
        CGContextAddLineToPoint(context, view_rect.size.width, baseline_y);
        CGContextStrokePath(context);
    }
    
    float ypos = 0.0f;
    NSInteger group_count = _scener.memberOf.count;
    float* right_edge_of_line = new float[group_count];
    for (int i=0; i<group_count; i++)
        right_edge_of_line[i] = 0.0f;
    int current_line = 0;
    
    // Draw each group membership
    for (CSMemberOfItem* member in _scener.memberOf)
    {
        float left_edge = left_margin;
        
        BOOL active = [member.status isEqualToString:@"active"];
        BOOL inactive = [member.status isEqualToString:@"inactive"];
        
        NSInteger joinYear = _earliestYear;
        if (member.joinDate != nil && member.joinDate.year != 0)
            joinYear = member.joinDate.year;
        
        left_edge += 96.0f * float(joinYear - _earliestYear);
        
        if (member.joinDate != nil && member.joinDate.month != 0)
            left_edge += 8.0 * float(member.joinDate.month - 1);
        
        if (member.joinDate != nil && member.joinDate.day != 0)
            left_edge += 8.0f/31.0f * float(member.joinDate.day - 1);

        // Find row for this group
        for (int i=0; i<group_count; i++)
        {
            if (right_edge_of_line[i] < left_edge)
            {
                current_line = i;
                break;
            }
        }
        
        ypos = float(current_line) * 18.0f;
        _maxHeight = MAX(_maxHeight, ypos);
        float right_edge = left_margin;
        
        NSInteger leaveYear = _latestYear;
        if (member.leaveDate != nil && member.leaveDate.year != 0)
            leaveYear = member.leaveDate.year;
        
        if (leaveYear == joinYear && (member.leaveDate != nil || member.leaveDate.month == 1) && (member.leaveDate != nil && member.leaveDate.day == 1))
            leaveYear += 1;
        
        right_edge += 96.0f * float(leaveYear - _earliestYear);
        
        if (member.leaveDate != nil && member.leaveDate.month != 0)
            right_edge += 8.0 * float(member.leaveDate.month - 1);
        
        if (member.leaveDate != nil && member.leaveDate.day != 0)
            right_edge += 8.0f/31.0f * float(member.leaveDate.day - 1);
        
        char* group_name = NULL;
        if ([member.group.name canBeConvertedToEncoding:NSMacOSRomanStringEncoding])
            group_name = (char*)[member.group.name cStringUsingEncoding:NSMacOSRomanStringEncoding];
        else
        {
            NSData* data = [member.group.name dataUsingEncoding:NSMacOSRomanStringEncoding allowLossyConversion:YES];
            NSUInteger length = [data length];
            group_name = new char[length+1];
            [data getBytes:group_name length:length];
            group_name[length] = 0;
        }

        if (right_edge == left_edge)
            right_edge = left_edge + 96.0f;
        
        CGRect group_rect = CGRectMake(left_edge, top_margin + ypos, right_edge - left_edge, 16.0f);
        
        float xpos = left_edge + 4.0f;
        float text_width = 0.0f;
        float text_xpos = xpos;
        if (context != NULL)
        {
            CGContextSetTextDrawingMode(context, kCGTextInvisible);
            CGContextShowTextAtPoint(context, text_xpos, top_margin + ypos + 12.0f, group_name, strlen(group_name));		
            CGPoint endPoint = CGContextGetTextPosition(context);
            text_width = (endPoint.x - text_xpos) + 4.0f;
        }
        else
        {
            CGSize size = [member.group.name sizeWithFont:[UIFont fontWithName:@"Helvetica" size:12.0f] constrainedToSize:CGSizeMake(10000.0f, 10000.0f)];
            text_width = size.width + 4.0f;
        }
        
        BOOL group_name_outside_of_box = NO;
        
        if ((text_xpos + text_width) > right_edge)
        {
            text_xpos = right_edge + 4.0f;
            right_edge = text_xpos + text_width;
            group_name_outside_of_box = YES;
        }
        
        right_edge_of_line[current_line] = right_edge;

        if (context != NULL)
        {
            CGColorSpaceRef baseSpace = CGColorSpaceCreateDeviceRGB();

            CGFloat active_colors[] = { 1.0f, 1.0f, 1.0f, 1.0f,  0.8f, 0.8f, 0.8f, 1.0f };
            CGGradientRef active_gradient = CGGradientCreateWithColorComponents(baseSpace, active_colors, NULL, 2);

            CGFloat inactive_colors[] = { 0.6f, 0.6f, 0.6f, 1.0f,  0.4f, 0.4f, 0.4f, 1.0f };
            CGGradientRef inactive_gradient = CGGradientCreateWithColorComponents(baseSpace, inactive_colors, NULL, 2);

            CGFloat ex_colors[] = { 0.2f, 0.2f, 0.2f, 1.0f,  0.0f, 0.0f, 0.0f, 1.0f };
            CGGradientRef ex_gradient = CGGradientCreateWithColorComponents(baseSpace, ex_colors, NULL, 2);
            
            CGColorSpaceRelease(baseSpace);
            
            if (active && !inactive)
            {
                sDrawGradientRect(context, group_rect, active_gradient);
                CGContextSetRGBStrokeColor(context, 0.5f, 0.5f, 0.5f, 1.0f);
                CGContextSetRGBFillColor(context, 1.0f, 1.0f, 1.0f, 1.0f);
            }
            else if (!active && inactive)
            {
                sDrawGradientRect(context, group_rect, inactive_gradient);
                CGContextSetRGBStrokeColor(context, 0.2f, 0.2f, 0.2f, 1.0f);
                CGContextSetRGBFillColor(context, 0.5f, 0.5f, 0.5f, 1.0f);
            }
            else if (!active && !inactive)
            {
                sDrawGradientRect(context, group_rect, ex_gradient);
                CGContextSetRGBStrokeColor(context, 0.5f, 0.5f, 0.5f, 1.0f);
                CGContextSetRGBFillColor(context, 0.1f, 0.1f, 0.1f, 1.0f);
            }
            
            CGContextStrokeRect(context, group_rect);

            // Clear out background before drawing group name outside of box
            if (group_name_outside_of_box)
            {
                CGRect group_label_rect = CGRectMake(text_xpos - 1.0f, top_margin + ypos, text_width, 14.0f);
                
                CGContextSetRGBFillColor(context, 0.85f, 0.85f, 0.85f, 1.0f);
                CGContextFillRect(context, group_label_rect);
            }
            
            if ((active && !inactive) || group_name_outside_of_box)
            {
                CGContextSetRGBStrokeColor(context, 0.0f, 0.0f, 0.0f, 1.0f);
                CGContextSetRGBFillColor(context, 0.0f, 0.0f, 0.0f, 1.0f);
            }
            else
            {
                CGContextSetRGBStrokeColor(context, 1.0f, 1.0f, 1.0f, 1.0f);
                CGContextSetRGBFillColor(context, 1.0f, 1.0f, 1.0f, 1.0f);
            }
            
            CGContextSetTextDrawingMode(context, kCGTextFill);
            CGContextShowTextAtPoint(context, text_xpos, top_margin + ypos + 12.0f, group_name, strlen(group_name));
            
            CGGradientRelease(active_gradient);
            CGGradientRelease(inactive_gradient);
            CGGradientRelease(ex_gradient);
        }
        
        _maxWidth = MAX(_maxWidth, right_edge);
    }
    
    _maxHeight += sLegendHeight;

    if (context != NULL)
    {
        CGColorSpaceRef baseSpace = CGColorSpaceCreateDeviceRGB();
        
        CGFloat active_colors[] = { 1.0f, 1.0f, 1.0f, 1.0f,  0.8f, 0.8f, 0.8f, 1.0f };
        CGGradientRef active_gradient = CGGradientCreateWithColorComponents(baseSpace, active_colors, NULL, 2);
        
        CGFloat inactive_colors[] = { 0.6f, 0.6f, 0.6f, 1.0f,  0.4f, 0.4f, 0.4f, 1.0f };
        CGGradientRef inactive_gradient = CGGradientCreateWithColorComponents(baseSpace, inactive_colors, NULL, 2);
        
        CGFloat ex_colors[] = { 0.2f, 0.2f, 0.2f, 1.0f,  0.0f, 0.0f, 0.0f, 1.0f };
        CGGradientRef ex_gradient = CGGradientCreateWithColorComponents(baseSpace, ex_colors, NULL, 2);
        
        CGColorSpaceRelease(baseSpace);
        
        CGContextSetTextDrawingMode(context, kCGTextFill);
        
        float legend_item_width = 192.0f;
        float legend_ypos = view_rect.size.height - sLegendHeight + 4.5f;
        
        // Active legend item
        CGRect legend_rect = CGRectMake(left_margin, legend_ypos, legend_item_width/3.0f, 16.0f);
        sDrawGradientRect(context, legend_rect, active_gradient);
        CGContextSetRGBStrokeColor(context, 0.5f, 0.5f, 0.5f, 1.0f);
        CGContextSetRGBFillColor(context, 1.0f, 1.0f, 1.0f, 1.0f);
        CGContextStrokeRect(context, legend_rect);
        
        // Inactive legend item
        legend_rect = CGRectMake(left_margin + 1.0f*legend_item_width, legend_ypos, legend_item_width/3.0f, 16.0f);
        sDrawGradientRect(context, legend_rect, inactive_gradient);
        CGContextSetRGBStrokeColor(context, 0.2f, 0.2f, 0.2f, 1.0f);
        CGContextSetRGBFillColor(context, 0.5f, 0.5f, 0.5f, 1.0f);
        CGContextStrokeRect(context, legend_rect);
        
        // Exactive legend item
        legend_rect = CGRectMake(left_margin + 2.0f*legend_item_width, legend_ypos, legend_item_width/3.0f, 16.0f);
        sDrawGradientRect(context, legend_rect, ex_gradient);
        CGContextSetRGBStrokeColor(context, 0.5f, 0.5f, 0.5f, 1.0f);
        CGContextSetRGBFillColor(context, 0.1f, 0.1f, 0.1f, 1.0f);
        CGContextStrokeRect(context, legend_rect);
        
        // legend text
        CGContextSetRGBStrokeColor(context, 0.0f, 0.0f, 0.0f, 1.0f);
        CGContextSetRGBFillColor(context, 0.0f, 0.0f, 0.0f, 1.0f);
        
        const char* legend_text = "active member";
        CGContextShowTextAtPoint(context, left_margin + 4.0f + legend_item_width/3.0f, legend_ypos + 12.0f, legend_text, strlen(legend_text));
        
        legend_text = "inactive member";
        CGContextShowTextAtPoint(context, left_margin + 4.0f + 1.0f*legend_item_width + legend_item_width/3.0f, legend_ypos + 12.0f, legend_text, strlen(legend_text));
        
        legend_text = "ex member";
        CGContextShowTextAtPoint(context, left_margin + 4.0f + 2.0f*legend_item_width + legend_item_width/3.0f, legend_ypos + 12.0f, legend_text, strlen(legend_text));
        
        CGGradientRelease(active_gradient);
        CGGradientRelease(inactive_gradient);
        CGGradientRelease(ex_gradient);
    }
    
    _maxWidth = MAX(_maxWidth, sLegendWidth);
    
    delete[] right_edge_of_line;
    _maxWidth += 32.0f;
    _maxHeight += 18.0f;
}


#pragma mark -
#pragma mark UIScrollViewDelegate


// ----------------------------------------------------------------------------
- (UIView *)viewForZoomingInScrollView:(UIScrollView *)scrollView
// ----------------------------------------------------------------------------
{
    return self;
}


@end
