//
//  StartPageViewController.m
//  C64SceneBrowser
//
//  Created by Andreas Varga on 07.07.12.
//
//

#import <QuartzCore/QuartzCore.h>

#import "StartPageViewController.h"
#import "MasterViewController.h"
#import "DetailViewController.h"
#import "EmulationViewController.h"
#import "AppDelegate.h"
#import "OHAttributedLabel.h"
#import "NSAttributedString+Attributes.h"
#import "CSDatabaseCache.h"
#import "UIImageView+WebCache.h"
#import "NSString+HTML.h"
#import "SSLoadingView.h"


@implementation StartPageViewController

@synthesize selectedOverviewItem = _selectedOverviewItem;
@synthesize selectedReleaseItem = _selectedReleaseItem;
@synthesize selectedScenerItem = _selectedScenerItem;
@synthesize isAppearing = _isAppearing;


// ----------------------------------------------------------------------------
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
// ----------------------------------------------------------------------------
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self)
    {
    }
    return self;
}


// ----------------------------------------------------------------------------
- (void)viewDidLoad
// ----------------------------------------------------------------------------
{
    [super viewDidLoad];

    NSString* sceneNewsUrl = @"http://csdb.dk/rss/scenenews.php";
    NSMutableURLRequest* request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:sceneNewsUrl] cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:60.0];
    _sceneNewsData = [NSMutableData data];
    _sceneNewsUrlConnection = [[NSURLConnection alloc] initWithRequest:request delegate:self];

    [self loadLatestComments];
    
    _newsItemLabel.alpha = 0.0f;

    _pageControlBeingUsed = NO;
    _isAppearing = NO;
}


// ----------------------------------------------------------------------------
- (void)didReceiveMemoryWarning
// ----------------------------------------------------------------------------
{
    [super didReceiveMemoryWarning];
}


// ----------------------------------------------------------------------------
-(UIStatusBarStyle) preferredStatusBarStyle
// ----------------------------------------------------------------------------
{
    return UIStatusBarStyleLightContent;
}


// ----------------------------------------------------------------------------
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
// ----------------------------------------------------------------------------
{
	return YES;
}


// ----------------------------------------------------------------------------
- (void) viewWillDisappear:(BOOL)animated
// ----------------------------------------------------------------------------
{
    [super viewWillDisappear:animated];
    
    [_newsItemFadeTimer invalidate];
    _newsItemFadeTimer = nil;
}


// ----------------------------------------------------------------------------
- (void) viewWillAppear:(BOOL)animated
// ----------------------------------------------------------------------------
{
    _isAppearing = YES;
    
    [super viewWillAppear:animated];
    
    _newsItemFadeTimer = [NSTimer scheduledTimerWithTimeInterval:5.0f target:self selector:@selector(fadeOutNewsItem) userInfo:nil repeats:NO];

    if (_loadingView != nil)
        return;

    [self scrollToPage:_pageControl.currentPage];
    
    if (_commentItems.count > 0)
        [self updateLatestComments];
}


// ----------------------------------------------------------------------------
- (void) viewDidAppear:(BOOL)animated
// ----------------------------------------------------------------------------
{
    _isAppearing = NO;
}



// ----------------------------------------------------------------------------
- (void) prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
// ----------------------------------------------------------------------------
{
    if ([[segue identifier] isEqualToString:@"ShowDetailView"])
    {
        DetailViewController* detailViewController = [segue destinationViewController];
        //NSLog(@"detailViewController: %@", detailViewController);
        detailViewController.overviewItem = _selectedOverviewItem;
        
        AppDelegate* appDelegate = (AppDelegate*)[[UIApplication sharedApplication] delegate];
        if (appDelegate.masterPopoverController != nil)
            [appDelegate.masterPopoverController dismissPopoverAnimated:YES];
    }
    else if ([[segue identifier] isEqualToString:@"ShowReleaseDetail"])
    {
        ReleaseViewController* releaseViewController = [segue destinationViewController];
        releaseViewController.releaseItem = _selectedReleaseItem;
        
        if (self.toolbarItems.count == 1)
        {
            NSMutableArray* toolbarItems = [NSMutableArray arrayWithObject:[self.toolbarItems objectAtIndex:0]];
            releaseViewController.toolbarItems = [toolbarItems arrayByAddingObjectsFromArray:releaseViewController.toolbarItems];
        }
    }
    else if ([[segue identifier] isEqualToString:@"ShowScenerDetail"])
    {
        ScenerViewController* scenerViewController = [segue destinationViewController];
        scenerViewController.scenerItem = _selectedScenerItem;
    }
}


// ----------------------------------------------------------------------------
- (void) didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
// ----------------------------------------------------------------------------
{
    if (_loadingView != nil)
        return;
    
    [self scrollToPage:_pageControl.currentPage];

    if (_commentItems.count > 0)
        [self updateLatestComments];
}


// ----------------------------------------------------------------------------
- (void) addMasterPopoverButton:(UIBarButtonItem*)barButtonItem
// ----------------------------------------------------------------------------
{
    NSMutableArray* toolbarItems = [NSMutableArray arrayWithObject:barButtonItem];
    self.toolbarItems = [toolbarItems arrayByAddingObjectsFromArray:self.toolbarItems];
}


// ----------------------------------------------------------------------------
- (void) removeMasterPopoverButton:(UIBarButtonItem*)barButtonItem
// ----------------------------------------------------------------------------
{
    if (self.toolbarItems.count > 0 && [self.toolbarItems objectAtIndex:0] == barButtonItem)
        self.toolbarItems = [self.toolbarItems subarrayWithRange:NSMakeRange(1, self.toolbarItems.count-1)];
}


// ----------------------------------------------------------------------------
- (void) updateSceneNews
// ----------------------------------------------------------------------------
{
    _newsItemLabel.text = [_sceneNewsItems objectAtIndex:_nextNewsItemIndex];

    _nextNewsItemIndex++;
    if (_nextNewsItemIndex >= _sceneNewsItems.count)
        _nextNewsItemIndex = 0;
    
    // Fade in
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:1.0f];
    [UIView setAnimationDelay:0.0f];
    [UIView setAnimationCurve:UIViewAnimationCurveLinear];

    _newsItemLabel.alpha = 1.0f;

    [UIView commitAnimations];

    // Schedule change after 5 seconds
    if (_newsItemFadeTimer != nil)
        [_newsItemFadeTimer invalidate];
    _newsItemFadeTimer = [NSTimer scheduledTimerWithTimeInterval:5.0f target:self selector:@selector(fadeOutNewsItem) userInfo:nil repeats:NO];
}


// ----------------------------------------------------------------------------
- (void) fadeOutNewsItem
// ----------------------------------------------------------------------------
{
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:2.0f];
    [UIView setAnimationDelay:0.0f];
    [UIView setAnimationCurve:UIViewAnimationCurveLinear];
    [UIView setAnimationDelegate:self];
    [UIView setAnimationDidStopSelector:@selector(updateSceneNews)];
    
    _newsItemLabel.alpha = 0.0f;
    
    [UIView commitAnimations];
}


// ----------------------------------------------------------------------------
- (void) loadLatestComments
// ----------------------------------------------------------------------------
{
    _loadingView = [[SSLoadingView alloc] initWithFrame:_scrollView.frame];
    _loadingView.autoresizingMask = _scrollView.autoresizingMask;
    [[_scrollView superview] addSubview:_loadingView];
    _scrollView.userInteractionEnabled = NO;
    
    NSString* commentsUrl = @"http://csdb.dk/rss/latestcomments.php";
    NSMutableURLRequest* request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:commentsUrl] cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:60.0];
    _commentsData = [NSMutableData data];
    _commentsUrlConnection = [[NSURLConnection alloc] initWithRequest:request delegate:self];
}


// ----------------------------------------------------------------------------
- (void) updateLatestComments
// ----------------------------------------------------------------------------
{
    if (_loadingView != nil)
    {
        [_loadingView removeFromSuperview];
        _loadingView = nil;
    }
    
    _scrollView.userInteractionEnabled = YES;
    
    [[_scrollView subviews] makeObjectsPerformSelector:@selector(removeFromSuperview)];
    _commentImageViews = [NSMutableArray arrayWithCapacity:_commentItems.count];
    
    for (int i=0; i<_commentItems.count; i++)
    {
        FeaturedComment* comment = [_commentItems objectAtIndex:i];
        CSReleaseItem* releaseItem = comment.releaseItem;

        CGRect frame;
        frame.origin.x = _scrollView.frame.size.width * i;
        frame.origin.y = 0;
        frame.size = _scrollView.frame.size;
        
        UIView* subview = [[UIView alloc] initWithFrame:frame];
        subview.autoresizesSubviews = YES;
        subview.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        
        UIImageView* screenShotView = [[UIImageView alloc] initWithFrame:CGRectMake(0.0f, 0.0f, 384.0f, 272.0f)];
        [[screenShotView subviews] makeObjectsPerformSelector:@selector(removeFromSuperview)];
        screenShotView.contentMode = UIViewContentModeCenter;
        screenShotView.backgroundColor = [UIColor blackColor];
        screenShotView.userInteractionEnabled = YES;
        screenShotView.clipsToBounds = YES;
        if (releaseItem != nil && releaseItem.screenshot != nil && releaseItem.screenshot.length > 0)
        {
            NSURL* url = [NSURL URLWithString:releaseItem.screenshot];
            [screenShotView setImageWithURL:url placeholderImage:nil];
        }
        else
        {
            screenShotView.image = nil;
            
            if (releaseItem.screenshot == nil || releaseItem.screenshot.length == 0)
            {
                UILabel* noImageLabel = [[UILabel alloc] initWithFrame:screenShotView.bounds];
                noImageLabel.text = @"No screenshot available";
                noImageLabel.textAlignment = NSTextAlignmentCenter;
                noImageLabel.numberOfLines = 0;
                noImageLabel.backgroundColor = [UIColor clearColor];
                noImageLabel.textColor = [UIColor whiteColor];
                noImageLabel.font = [UIFont boldSystemFontOfSize:24.0f];
                [screenShotView addSubview:noImageLabel];
            }
        }
        
        UITapGestureRecognizer* tapRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tappedImage:)];
        [screenShotView addGestureRecognizer:tapRecognizer];
        
        [subview addSubview:screenShotView];
        [_commentImageViews addObject:screenShotView];
        
        UILabel* titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(392.0f, 0.0f, _scrollView.frame.size.width - 403.0f, 31.0f)];
        titleLabel.text = comment.title;
        titleLabel.font = [UIFont systemFontOfSize:24.0f];
        titleLabel.adjustsFontSizeToFitWidth = YES;
        titleLabel.minimumScaleFactor = 0.8f;
        titleLabel.textAlignment = NSTextAlignmentLeft;
        titleLabel.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        titleLabel.textColor = [UIColor whiteColor];
        titleLabel.backgroundColor = [UIColor clearColor];
        [subview addSubview:titleLabel];
        
        UILabel* authorLabel = [[UILabel alloc] initWithFrame:CGRectMake(392.0f, 30.0f, _scrollView.frame.size.width - 535.0f, 27.0f)];
        authorLabel.text = comment.author;
        authorLabel.font = [UIFont systemFontOfSize:18.0f];
        authorLabel.adjustsFontSizeToFitWidth = YES;
        authorLabel.minimumScaleFactor = 0.88f;
        authorLabel.textAlignment = NSTextAlignmentLeft;
        authorLabel.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        authorLabel.textColor = [UIColor whiteColor];
        authorLabel.backgroundColor = [UIColor clearColor];
        [subview addSubview:authorLabel];
        
        UILabel* dateLabel = [[UILabel alloc] initWithFrame:CGRectMake(564.0f, 30.0f, _scrollView.frame.size.width - 575.0f, 27.0f)];
        dateLabel.text = comment.date;
        dateLabel.font = [UIFont systemFontOfSize:16.0f];
        dateLabel.adjustsFontSizeToFitWidth = NO;
        dateLabel.textAlignment = NSTextAlignmentRight;
        dateLabel.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin;
        dateLabel.textColor = [UIColor grayColor];
        dateLabel.backgroundColor = [UIColor clearColor];
        [subview addSubview:dateLabel];

        CGFloat width = _scrollView.frame.size.width - 403.0f;

        CGRect rect = [comment.description boundingRectWithSize:CGSizeMake(width, 10000.0f) options:NSStringDrawingUsesLineFragmentOrigin attributes:[NSDictionary dictionaryWithObjectsAndKeys:[UIFont systemFontOfSize:16.0f], NSFontAttributeName, nil] context:NULL];
        rect.size.width = width;

        UIScrollView* commentScrollView = [[UIScrollView alloc] initWithFrame:CGRectMake(392.0f, 60.0f, width, 210.0f)];
        commentScrollView.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        commentScrollView.scrollEnabled = YES;
        commentScrollView.contentSize = rect.size;
        
        OHAttributedLabel* commentLabel = [[OHAttributedLabel alloc] initWithFrame:rect];
        commentLabel.text = comment.description;
        commentLabel.font = [UIFont systemFontOfSize:16.0f];
        commentLabel.textAlignment = NSTextAlignmentLeft;
        commentLabel.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        commentLabel.textColor = [UIColor whiteColor];
        commentLabel.backgroundColor = [UIColor clearColor];
        commentLabel.lineBreakMode = NSLineBreakByWordWrapping;
        commentLabel.linkColor = [UIColor colorWithWhite:0.7f alpha:1.0f];
        commentLabel.highlightedLinkColor = [UIColor colorWithWhite:0.9f alpha:1.0f];
        //commentLabel.numberOfLines = 0;
        [commentScrollView addSubview:commentLabel];

        [subview addSubview:commentScrollView];
        
        [_scrollView addSubview:subview];
    }

    _scrollView.autoresizesSubviews = YES;
    _scrollView.contentSize = CGSizeMake(_scrollView.frame.size.width * _commentItems.count, _scrollView.frame.size.height);
}


// ----------------------------------------------------------------------------
- (void) scrollToPage:(NSUInteger)pageIndex
// ----------------------------------------------------------------------------
{
    // update the scroll view to the appropriate page
    CGRect frame;
    frame.origin.x = _scrollView.frame.size.width * pageIndex;
    frame.origin.y = 0;
    frame.size = _scrollView.frame.size;
    [_scrollView scrollRectToVisible:frame animated:YES];
    
    _pageControlBeingUsed = YES;
}


// ----------------------------------------------------------------------------
- (IBAction) changePage
// ----------------------------------------------------------------------------
{
    [self scrollToPage:_pageControl.currentPage];
}


// ----------------------------------------------------------------------------
- (void) tappedImage:(id)sender
// ----------------------------------------------------------------------------
{
    UIImageView* imageView = (UIImageView*)[sender view];
    NSUInteger index = [_commentImageViews indexOfObject:imageView];
    if (index == NSNotFound)
        return;

    FeaturedComment* comment = [_commentItems objectAtIndex:index];
    _selectedReleaseItem = comment.releaseItem;
    if (_selectedReleaseItem != nil)
        [self performSegueWithIdentifier:@"ShowReleaseDetail" sender:self];
}


// ----------------------------------------------------------------------------
- (IBAction) refresh:(id)sender
// ----------------------------------------------------------------------------
{
    [self loadLatestComments];
    _pageControl.numberOfPages = 1;
    _pageControl.currentPage = 0;
    [self scrollToPage:0];
}


#pragma mark -

// ----------------------------------------------------------------------------
- (void) connection:(NSURLConnection*)connection didReceiveResponse:(NSURLResponse*)response
// ----------------------------------------------------------------------------
{
    if (connection == _sceneNewsUrlConnection)
    {
        [_sceneNewsData setLength:0];
        return;
    }
    else if (connection == _commentsUrlConnection)
    {
        [_commentsData setLength:0];
        return;
    }
}


// ----------------------------------------------------------------------------
- (void) connection:(NSURLConnection*)connection didReceiveData:(NSData*)data
// ----------------------------------------------------------------------------
{
    if (connection == _sceneNewsUrlConnection)
    {
        [_sceneNewsData appendData:data];
        return;
    }
    else if (connection == _commentsUrlConnection)
    {
        [_commentsData appendData:data];
        return;
    }
}


// ----------------------------------------------------------------------------
- (void) connection:(NSURLConnection*)connection didFailWithError:(NSError*)error
// ----------------------------------------------------------------------------
{

}


// ----------------------------------------------------------------------------
- (void) connectionDidFinishLoading:(NSURLConnection*)connection
// ----------------------------------------------------------------------------
{
    if (connection == _sceneNewsUrlConnection && [_sceneNewsData length] > 0)
    {
        //NSLog(@"Scene news download finished\n");
        
        NSError* error;
        TBXML* tbxml = [TBXML tbxmlWithXMLData:_sceneNewsData error:&error];
        if (tbxml == NULL || tbxml.rootXMLElement == NULL)
            return;
        
        TBXMLElement* channelElement = [TBXML childElementNamed:@"channel" parentElement:tbxml.rootXMLElement];
        if (channelElement == NULL)
            return;
        
        _sceneNewsItems = [NSMutableArray array];
        TBXMLElement* itemElement = [TBXML childElementNamed:@"item" parentElement:channelElement];
        while (itemElement != NULL)
        {
            TBXMLElement* titleElement = [TBXML childElementNamed:@"title" parentElement:itemElement];
            NSString* title = @"";
            if (titleElement != NULL)
                title = [TBXML textForElement:titleElement withStringEncoding:NSUTF8StringEncoding];

            TBXMLElement* dateElement = [TBXML childElementNamed:@"pubDate" parentElement:itemElement];
            NSString* date = @"";
            if (dateElement != NULL)
            {
                date = [TBXML textForElement:dateElement withStringEncoding:NSUTF8StringEncoding];
                date = [date substringWithRange:NSMakeRange(5, 11)];
            }
            
            NSString* newsItemText = [NSString stringWithFormat:@"%@ - %@", date, title];
            [_sceneNewsItems addObject:newsItemText];
            
            itemElement = [TBXML nextSiblingNamed:@"item" searchFromElement:itemElement];
        }
        
        //NSLog(@"Found %d scene news", _sceneNewsItems.count);
        
        if (_sceneNewsItems.count > 0)
        {
            _nextNewsItemIndex = 0;
            [self updateSceneNews];
        }
    }
    else if (connection == _commentsUrlConnection && [_commentsData length] > 0)
    {
        //NSLog(@"Latest comments download finished\n");
        
        NSError* error;
        TBXML* tbxml = [TBXML tbxmlWithXMLData:_commentsData error:&error];
        if (tbxml == NULL || tbxml.rootXMLElement == NULL)
            return;
        
        TBXMLElement* channelElement = [TBXML childElementNamed:@"channel" parentElement:tbxml.rootXMLElement];
        if (channelElement == NULL)
            return;

        _commentItems = [NSMutableArray array];
        BOOL firstItemIsAlreadyCached = NO;
        TBXMLElement* itemElement = [TBXML childElementNamed:@"item" parentElement:channelElement];
        while (itemElement != NULL && _commentItems.count < 30)
        {
            TBXMLElement* titleElement = [TBXML childElementNamed:@"title" parentElement:itemElement];
            NSString* title = @"";
            if (titleElement != NULL)
            {
                title = [TBXML textForElement:titleElement withStringEncoding:NSUTF8StringEncoding];
                title = [title substringWithRange:NSMakeRange(0, title.length-4)];
                title = [title stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
                title = [title stringByDecodingHTMLEntities];
            }

            TBXMLElement* descriptionElement = [TBXML childElementNamed:@"description" parentElement:itemElement];
            NSString* description = @"";
            if (descriptionElement != NULL)
            {
                description = [TBXML textForElement:descriptionElement withStringEncoding:NSUTF8StringEncoding];
                description = [description stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
                description = [description stringByDecodingHTMLEntities];
                description = [description stringByConvertingHTMLToPlainText];
            }

            TBXMLElement* authorElement = [TBXML childElementNamed:@"author" parentElement:itemElement];
            NSString* author = @"";
            if (authorElement != NULL)
            {
                author = [TBXML textForElement:authorElement withStringEncoding:NSUTF8StringEncoding];
                author = [author stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
                author = [author stringByDecodingHTMLEntities];
            }

            TBXMLElement* dateElement = [TBXML childElementNamed:@"pubDate" parentElement:itemElement];
            NSString* date = @"";
            if (dateElement != NULL)
            {
                date = [TBXML textForElement:dateElement withStringEncoding:NSUTF8StringEncoding];
                date = [date substringWithRange:NSMakeRange(0, 16)];
            }

            TBXMLElement* guidElement = [TBXML childElementNamed:@"guid" parentElement:itemElement];
            NSString* url = @"";
            if (guidElement != NULL)
            {
                url = [TBXML textForElement:guidElement withStringEncoding:NSUTF8StringEncoding];
                if ([url containsString:@"/release/"])
                {
                    NSRange idRange = [url rangeOfString:@"id="];
                    NSRange ampersandRange = [url rangeOfString:@"&amp"];
                    if (idRange.location != NSNotFound && ampersandRange.location != NSNotFound)
                    {
                        NSRange releaseIdRange = NSMakeRange(idRange.location + 3, ampersandRange.location - (idRange.location + 3));
                        NSString* releaseIdString = [url substringWithRange:releaseIdRange];
                        
                        CSReleaseItem* releaseItem = (CSReleaseItem*)[[CSDatabaseCache sharedInstance] itemForType:CSDB_RELEASE withIdentifier:releaseIdString];
                        if (_commentItems.count == 0 && releaseItem.cached_depth < 1 && releaseItem.requested_depth < 1)
                            [releaseItem cacheAndNotify:self withDepth:1];
                        else
                            firstItemIsAlreadyCached = YES;

                        FeaturedComment* comment = [[FeaturedComment alloc] init];
                        comment.title = title;
                        comment.author = author;
                        comment.date = date;
                        comment.description = description; //@"This is a test of a comment with a link to http://www.google.com/ and let's check if the parsing works!";
                        comment.releaseItem = releaseItem;
                        
                        [_commentItems addObject:comment];
                    }
                }
            }
            
            itemElement = [TBXML nextSiblingNamed:@"item" searchFromElement:itemElement];
        }
        
        _pageControl.numberOfPages = _commentItems.count;
        //NSLog(@"Found %d comments", _commentItems.count);
        
        if (firstItemIsAlreadyCached)
            [self updateLatestComments];
    }
}


#pragma mark - CSDatabaseItemDelegate protocol


// ----------------------------------------------------------------------------
- (void) databaseItemIsCached:(CSDatabaseItem*)item
// ----------------------------------------------------------------------------
{
    // Check if item is first in list
    FeaturedComment* comment = [_commentItems objectAtIndex:0];
    if (comment.releaseItem == item)
    {
        [self updateLatestComments];
    }
    else
    {
        int index = 0;
        for (FeaturedComment* comment in _commentItems)
        {
            UIImageView* imageView = [_commentImageViews objectAtIndex:index];
            if (comment.releaseItem == item)
            {
                if (comment.releaseItem.screenshot != nil && comment.releaseItem.screenshot.length > 0)
                {
                    NSURL* url = [NSURL URLWithString:comment.releaseItem.screenshot];
                    [imageView setImageWithURL:url placeholderImage:nil];
                }
                else
                    imageView.image = nil;
            }
            
            index++;
        }
    }

    // Find first item that needs caching
    FeaturedComment* next_comment = nil;
    for (next_comment in _commentItems)
    {
        if (next_comment.releaseItem.cached_depth < 1)
            break;
    }
    
    if (next_comment != nil)
        [next_comment.releaseItem cacheAndNotify:self withDepth:1];
}


// ----------------------------------------------------------------------------
- (void) cachingFailed
// ----------------------------------------------------------------------------
{

}

#pragma mark - Scroll view delegate


// ----------------------------------------------------------------------------
- (void)scrollViewDidScroll:(UIScrollView *)sender
// ----------------------------------------------------------------------------
{
    if (_pageControlBeingUsed)
        return;
    
    // Update the page when more than 50% of the previous/next page is visible
    CGFloat pageWidth = _scrollView.frame.size.width;
    int page = floor((_scrollView.contentOffset.x - pageWidth / 2) / pageWidth) + 1;
    _pageControl.currentPage = page;
}


- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
    _pageControlBeingUsed = NO;
}

- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView
{
    _pageControlBeingUsed = NO;
}

#pragma mark - Split view delegate


// ----------------------------------------------------------------------------
- (void) splitViewController:(UISplitViewController *)splitController willHideViewController:(UIViewController *)viewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController:(UIPopoverController *)popoverController
// ----------------------------------------------------------------------------
{
    // When master view is hidden, this will add the "Categories" button which can be used to show the master view popover
    barButtonItem.title = @"Categories";
    
    for (UIViewController* viewController in self.navigationController.viewControllers)
        [viewController addMasterPopoverButton:barButtonItem];
    
    AppDelegate* appDelegate = (AppDelegate*)[[UIApplication sharedApplication] delegate];
    appDelegate.masterPopoverController = popoverController;
    appDelegate.masterPopoverBarButtonItem = barButtonItem;
}


// ----------------------------------------------------------------------------
- (void) splitViewController:(UISplitViewController *)splitController willShowViewController:(UIViewController *)viewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
// ----------------------------------------------------------------------------
{
    // Called when the view is shown again in the split view, invalidating the button and popover controller.
    for (UIViewController* viewController in self.navigationController.viewControllers)
        [viewController removeMasterPopoverButton:barButtonItem];
    
    AppDelegate* appDelegate = (AppDelegate*)[[UIApplication sharedApplication] delegate];
    appDelegate.masterPopoverController = nil;
    appDelegate.masterPopoverBarButtonItem = nil;
}


@end

@implementation FeaturedComment

@synthesize title = _title;
@synthesize author = _author;
@synthesize date = _date;
@synthesize description = _description;
@synthesize releaseItem = _releaseItem;


@end
